# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.datastructuretools.TreeNode import TreeNode


class TreeContainer(TreeNode):
    r'''A tree container.

    Inner node in a generalized tree data structure.

    ::

        >>> a = datastructuretools.TreeContainer()
        >>> a
        TreeContainer()

    ::

        >>> b = datastructuretools.TreeNode()
        >>> a.append(b)
        >>> a
        TreeContainer(
            children=(
                TreeNode(),
                )
            )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_children',
        '_named_children',
        )

    ### INITIALIZER ###

    def __init__(self, children=None, name=None):
        TreeNode.__init__(self, name=name)
        self._children = []
        self._named_children = {}
        if children is None:
            pass
        elif isinstance(children, (list, tuple)):
            self.extend(children)
        else:
            message = 'can not instantiate {} with {!r}.'
            message = message.format(type(self), children)
            raise ValueError(message)

    ### SPECIAL METHODS ###

    def __contains__(self, argument):
        r'''True if argument is in container. Otherwise false:

        ::

            >>> container = datastructuretools.TreeContainer()
            >>> a = datastructuretools.TreeNode()
            >>> b = datastructuretools.TreeNode()
            >>> container.append(a)

        ::

            >>> a in container
            True

        ::

            >>> b in container
            False

        Returns true or false.
        '''
        for x in self._children:
            if x is argument:
                return True
        return False

    def __delitem__(self, i):
        r'''Deletes node `i` in tree container.

        ::

            >>> container = datastructuretools.TreeContainer()
            >>> leaf = datastructuretools.TreeNode()

        ::

            >>> container.append(leaf)
            >>> container.children == (leaf,)
            True

        ::

            >>> leaf.parent is container
            True

        ::

            >>> del(container[0])

        ::

            >>> container.children == ()
            True

        ::

            >>> leaf.parent is None
            True

        Return `None`.
        '''
        nodes = self[i]
        if not isinstance(nodes, list):
            nodes = [nodes]
        for node in nodes:
            node._set_parent(None)
        self._mark_entire_tree_for_later_update()

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeNode()
            >>> c = datastructuretools.TreeContainer()
            >>> d = datastructuretools.TreeNode()
            >>> e = datastructuretools.TreeNode()
            >>> f = datastructuretools.TreeNode()

        ::

            >>> a.extend([b, c, f])
            >>> c.extend([d, e])

        ::

            >>> a[0] is b
            True

        ::

            >>> a[1] is c
            True

        ::

            >>> a[2] is f
            True

        If `i` is a string, the container will attempt to
        return the single child node, at any depth, whose
        `name` matches `i`:

        ::

            >>> foo = datastructuretools.TreeContainer(name='foo')
            >>> bar = datastructuretools.TreeContainer(name='bar')
            >>> baz = datastructuretools.TreeNode(name='baz')
            >>> quux = datastructuretools.TreeNode(name='quux')

        ::

            >>> foo.append(bar)
            >>> bar.extend([baz, quux])

        ::

            >>> foo['bar'] is bar
            True

        ::

            >>> foo['baz'] is baz
            True

        ::

            >>> foo['quux'] is quux
            True

        Return `TreeNode` instance.
        '''
        if isinstance(argument, (int, slice)):
            return self._children.__getitem__(argument)
        elif isinstance(argument, str):
            children = self._named_children.__getitem__(argument)
            if 1 == len(children):
                return tuple(children)[0]
        raise ValueError(repr(argument))

    def __iter__(self):
        r'''Iterates tree container.

        Yields children of tree container.
        '''
        for child in self._children:
            yield child

    def __len__(self):
        r'''Returns nonnegative integer number of nodes in container.
        '''
        return len(self._children)

    def __setitem__(self, i, argument):
        r'''Sets `argument` in self at nonnegative integer index `i`, or set `argument`
        in self at slice i. Replace contents of `self[i]` with `argument`.
        Attach parentage to contents of `argument`, and detach parentage
        of any replaced nodes:

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeNode()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.parent is a
            True

        ::

            >>> a.children == (b,)
            True

        ::

            >>> a[0] = c

        ::

            >>> c.parent is a
            True


        ::

            >>> b.parent is None
            True

        ::

            >>> a.children == (c,)
            True

        Returns none.
        '''
        proper_parentage = self.proper_parentage

        if isinstance(i, int):
            assert isinstance(argument, self._node_class)
            old = self[i]
            assert argument not in proper_parentage
            old._set_parent(None)
            argument._set_parent(self)
            self._children.insert(i, argument)
        else:
            if isinstance(argument, TreeContainer):
                # Prevent mutating while iterating by copying.
                argument = argument[:]
            assert all(isinstance(x, self._node_class) for x in argument)
            if i.start == i.stop and i.start is not None \
                and i.stop is not None and i.start <= -len(self):
                start, stop = 0, 0
            else:
                start, stop, stride = i.indices(len(self))
            old = self[start:stop]
            for node in argument:
                assert node not in proper_parentage
            for node in old:
                node._set_parent(None)
            for node in argument:
                node._set_parent(self)
            self._children.__setitem__(slice(start, start), argument)
        self._mark_entire_tree_for_later_update()

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        template_names = names[:]
        for name in ('children',):
            if not getattr(self, name, None) and name in names:
                names.remove(name)
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=True,
            storage_format_kwargs_names=names,
            template_names=template_names,
            )

    ### PUBLIC METHODS ###

    def append(self, node):
        r'''Appends `node` to tree container.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeNode()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a
            TreeContainer()

        ::

            >>> a.append(b)
            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    )
                )

        ::

            >>> a.append(c)
            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    TreeNode(),
                    )
                )

        Returns none.
        '''
        self.__setitem__(
            slice(len(self), len(self)),
            [node]
            )

    def extend(self, argument):
        r'''Extendes `argument` against tree container.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeNode()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a
            TreeContainer()

        ::

            >>> a.extend([b, c])
            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    TreeNode(),
                    )
                )

        Returns none.
        '''
        self.__setitem__(
            slice(len(self), len(self)),
            argument
            )

    def index(self, node):
        r'''Indexes `node` in tree container.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeNode()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.extend([b, c])

        ::

            >>> a.index(b)
            0

        ::

            >>> a.index(c)
            1

        Returns nonnegative integer.
        '''
        for i, element in enumerate(self._children):
            if element is node:
                return i
        else:
            message = 'node {!r} not in {!r}.'
            message = message.format(node, self)
            raise ValueError(message)

    def insert(self, i, node):
        r'''Insert `node` in tree container at index `i`.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeNode()
            >>> c = datastructuretools.TreeNode()
            >>> d = datastructuretools.TreeNode()

        ::

            >>> a.extend([b, c])

        ::

            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    TreeNode(),
                    )
                )

        ::

            >>> a.insert(1, d)

        ::

            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    TreeNode(),
                    TreeNode(),
                    )
                )


        Return `None`.
        '''
        self.__setitem__(
            slice(i, i),
            [node]
            )

    def pop(self, i=-1):
        r'''Pops node `i` from tree container.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeNode()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.extend([b, c])

        ::

            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    TreeNode(),
                    )
                )

        ::

            >>> node = a.pop()

        ::

            >>> node == c
            True

        ::

            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    )
                )

        Returns node.
        '''
        node = self[i]
        del(self[i])
        return node

    def remove(self, node):
        r'''Remove `node` from tree container.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeNode()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.extend([b, c])

        ::

            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    TreeNode(),
                    )
                )

        ::

            >>> a.remove(b)

        ::

            >>> a
            TreeContainer(
                children=(
                    TreeNode(),
                    )
                )

        Returns none.
        '''
        i = self.index(node)
        del(self[i])

    ### PRIVATE PROPERTIES ###

    @property
    def _leaf_class(self):
        return TreeNode

    @property
    def _node_class(self):
        return TreeNode

    ### PUBLIC PROPERTIES ###

    @property
    def children(self):
        r'''Children of tree container.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()
            >>> d = datastructuretools.TreeNode()
            >>> e = datastructuretools.TreeContainer()

        ::

            >>> a.extend([b, c])
            >>> b.extend([d, e])

        ::

            >>> a.children == (b, c)
            True

        ::

            >>> b.children == (d, e)
            True

        ::

            >>> e.children == ()
            True

        Returns tuple of tree nodes.
        '''
        return tuple(self._children)

    @property
    def leaves(self):
        r'''Leaves of tree container.

        ::

            >>> a = datastructuretools.TreeContainer(name='a')
            >>> b = datastructuretools.TreeContainer(name='b')
            >>> c = datastructuretools.TreeNode(name='c')
            >>> d = datastructuretools.TreeNode(name='d')
            >>> e = datastructuretools.TreeContainer(name='e')

        ::

            >>> a.extend([b, c])
            >>> b.extend([d, e])

        ::

            >>> for leaf in a.leaves:
            ...     print(leaf.name)
            ...
            d
            e
            c

        Returns tuple.
        '''
        def recurse(node):
            result = []
            for child in node:
                if not hasattr(child, 'children'):
                    if isinstance(child, self._leaf_class):
                        result.append(child)
                elif not child.children:
                    if isinstance(child, self._leaf_class):
                        result.append(child)
                else:
                    result.extend(recurse(child))
            return result
        return tuple(recurse(self))

    @property
    def nodes(self):
        r'''The collection of tree nodes produced by iterating tree container
        depth-first.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()
            >>> d = datastructuretools.TreeNode()
            >>> e = datastructuretools.TreeContainer()

        ::

            >>> a.extend([b, c])
            >>> b.extend([d, e])

        ::

            >>> nodes = a.nodes
            >>> len(nodes)
            5

        ::

            >>> nodes[0] is a
            True

        ::

            >>> nodes[1] is b
            True

        ::

            >>> nodes[2] is d
            True

        ::

            >>> nodes[3] is e
            True

        ::

            >>> nodes[4] is c
            True

        Returns tuple.
        '''
        def recurse(container):
            result = []
            for child in container.children:
                result.append(child)
                if isinstance(child, datastructuretools.TreeContainer):
                    result.extend(recurse(child))
            return result
        from abjad.tools import datastructuretools
        result = [self] + recurse(self)
        return tuple(result)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ScaleDegree__initialize_by_accidental_and_number_01():

    degree = tonalanalysistools.ScaleDegree('flat', 2)
    assert degree.accidental == pitchtools.Accidental('flat')
    assert degree.number == 2



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection__set_component_parents_01():

    voice = Voice([])
    u = Voice("c'8 d'8 e'8 f'8")

    selection = u[:]
    selection._set_parents(voice)

    assert len(u) == 0

    "Container voice now assigned to selection."
    "But selection not in container voice."

    assert selection[0]._parent is voice
    assert selection[0] not in voice

    voice._music.extend(selection)

    "Selection now in container voice."

    assert inspect_(voice).is_well_formed()
    assert selection[0]._parent is voice
    assert selection[0] in voice


# -*- coding: utf-8 -*-
from abjad import *


def test_durationtools_Duration___add___01():
    r'''Adding two durations returns a third duration.
    '''

    duration = Duration(1, 2) + Duration(1, 3)
    assert isinstance(duration, Duration)


def test_durationtools_Duration___add___02():
    r'''Adding a duration and nonreduced fraction
    returns a nonreduced fraction.
    '''

    result = Duration(1, 4) + mathtools.NonreducedFraction(2, 8)
    assert isinstance(result, mathtools.NonreducedFraction)
    assert result.pair == (4, 8)

    result = mathtools.NonreducedFraction(2, 8) + Duration(1, 4)
    assert isinstance(result, mathtools.NonreducedFraction)
    assert result.pair == (4, 8)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_LilyPondCommand___init___01():
    r'''Initializes LilyPond command from command name.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, staff[:])
    command = indicatortools.LilyPondCommand(r'slurDotted')
    attach(command, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \slurDotted
            c'8 (
            d'8
            e'8
            f'8 )
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_indicatortools_LilyPondCommand___init___02():
    r'''Set LilyPond command from command name.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, staff[:])
    command = indicatortools.LilyPondCommand(r'slurUp')
    attach(command, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \slurUp
            c'8 (
            d'8
            e'8
            f'8 )
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_indicatortools_LilyPondCommand___init___03():
    r'''Initializes LilyPond command from string and format slot.
    '''

    command = indicatortools.LilyPondCommand('break', 'closing')
    assert isinstance(command, indicatortools.LilyPondCommand)


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval___add___01():

    i = pitchtools.NumberedInterval(3)
    j = pitchtools.NumberedInterval(14)

    assert i + j == pitchtools.NumberedInterval(17)
    assert j + i == pitchtools.NumberedInterval(17)


def test_pitchtools_NumberedInterval___add___02():

    i = pitchtools.NumberedInterval(3)
    j = pitchtools.NumberedInterval(14)

    assert i - j == pitchtools.NumberedInterval(-11)
    assert j - i == pitchtools.NumberedInterval(11)



# -*- coding: utf-8 -*-
import os
import platform
from abjad.tools import commandlinetools
from abjad.tools import stringtools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    expected_files = [
        'test_score/test_score/materials/.gitignore',
        'test_score/test_score/materials/__init__.py',
        'test_score/test_score/materials/test_material/__init__.py',
        'test_score/test_score/materials/test_material/definition.py',
        'test_score/test_score/materials/test_material/illustration.ly',
        'test_score/test_score/materials/test_material/illustration.pdf',
        ]

    if platform.system().lower() == 'windows':
        expected_files = [_.replace('/', os.path.sep) for _ in expected_files]

    expected_illustration_contents = stringtools.normalize(
        r'''
        \language "english"

        \header {
            tagline = ##f
        }

        \layout {}

        \paper {}

        \markup { "An example illustrable material." }
        '''
        )

    def test_lilypond_error(self):
        """
        Handle failing LilyPond rendering.
        """
        self.create_score()
        material_path = self.create_material('test_material')
        definition_path = material_path.joinpath('definition.py')
        with open(str(definition_path), 'w') as file_pointer:
            file_pointer.write(stringtools.normalize(r'''
            # -*- coding: utf-8 -*-
            from abjad.tools import lilypondfiletools


            test_material = lilypondfiletools.LilyPondFile.new()
            test_material.items.append(r'\this-does-not-exist')
            '''))
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', 'test_material']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_material' ...
            Illustrating test_score/materials/test_material/
                Importing test_score.materials.test_material.definition
                    Abjad runtime: ... second...
                Writing test_score/materials/test_material/illustration.ly ... OK!
                Writing test_score/materials/test_material/illustration.pdf ... Failed!
        '''.replace('/', os.path.sep))
        illustration_ly_path = material_path.joinpath('illustration.ly')
        assert illustration_ly_path.exists()
        self.compare_lilypond_contents(
            illustration_ly_path, stringtools.normalize(r'''
            \language "english"

            \header {}

            \layout {}

            \paper {}

            \this-does-not-exist
            '''))

    def test_missing_definition(self):
        """
        Handle missing definition.
        """
        self.create_score()
        material_path = self.create_material('test_material')
        definition_path = material_path.joinpath('definition.py')
        definition_path.unlink()
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', 'test_material']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_material' ...
            Illustrating test_score/materials/test_material/
                Importing test_score.materials.test_material.definition
        '''.replace('/', os.path.sep))

    def test_python_cannot_illustrate(self):
        """
        Handle un-illustrables.
        """
        self.create_score()
        material_path = self.create_material('test_material')
        definition_path = material_path.joinpath('definition.py')
        with open(str(definition_path), 'w') as file_pointer:
            file_pointer.write(stringtools.normalize(r'''
            # -*- coding: utf-8 -*-

            test_material = None
            '''))
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', 'test_material']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_material' ...
            Illustrating test_score/materials/test_material/
                Importing test_score.materials.test_material.definition
                Cannot illustrate material of type NoneType.
        '''.replace('/', os.path.sep))

    def test_python_error_on_illustrate(self):
        """
        Handle exceptions inside the Python module on __call__().
        """
        self.create_score()
        material_path = self.create_material('test_material')
        definition_path = material_path.joinpath('definition.py')
        with open(str(definition_path), 'w') as file_pointer:
            file_pointer.write(stringtools.normalize(r'''
            # -*- coding: utf-8 -*-
            from abjad.tools import abctools


            class Foo(object):
                def __illustrate__(self):
                    raise TypeError('This is fake.')

            test_material = Foo()
            '''))
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', 'test_material']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_material' ...
            Illustrating test_score/materials/test_material/
                Importing test_score.materials.test_material.definition
        '''.replace('/', os.path.sep))

    def test_python_error_on_import(self):
        """
        Handle exceptions inside the Python module on import.
        """
        self.create_score()
        material_path = self.create_material('test_material')
        definition_path = material_path.joinpath('definition.py')
        with open(str(definition_path), 'a') as file_pointer:
            file_pointer.write('\n\nfailure = 1 / 0\n')
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', 'test_material']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_material' ...
            Illustrating test_score/materials/test_material/
                Importing test_score.materials.test_material.definition
        '''.replace('/', os.path.sep))

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_all_materials(self, open_file_mock):
        self.create_score()
        self.create_material('material_one')
        self.create_material('material_two')
        self.create_material('material_three')
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', '*']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
            Illustration candidates: '*' ...
            Illustrating test_score/materials/material_one/
                Importing test_score.materials.material_one.definition
                    Abjad runtime: ... second...
                Writing test_score/materials/material_one/illustration.ly ... OK!
                Writing test_score/materials/material_one/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/materials/material_one/
            Illustrating test_score/materials/material_three/
                Importing test_score.materials.material_three.definition
                    Abjad runtime: ... second...
                Writing test_score/materials/material_three/illustration.ly ... OK!
                Writing test_score/materials/material_three/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/materials/material_three/
            Illustrating test_score/materials/material_two/
                Importing test_score.materials.material_two.definition
                    Abjad runtime: ... second...
                Writing test_score/materials/material_two/illustration.ly ... OK!
                Writing test_score/materials/material_two/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/materials/material_two/
        '''.replace('/', os.path.sep))
        assert self.materials_path.joinpath(
            'material_one',
            'illustration.pdf',
            ).exists()
        assert self.materials_path.joinpath(
            'material_two',
            'illustration.pdf',
            ).exists()
        assert self.materials_path.joinpath(
            'material_three',
            'illustration.pdf',
            ).exists()

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_filtered_materials(self, open_file_mock):
        self.create_score()
        self.create_material('material_one')
        self.create_material('material_two')
        self.create_material('material_three')
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', 'material_t*']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
            Illustration candidates: 'material_t*' ...
            Illustrating test_score/materials/material_three/
                Importing test_score.materials.material_three.definition
                    Abjad runtime: ... second...
                Writing test_score/materials/material_three/illustration.ly ... OK!
                Writing test_score/materials/material_three/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/materials/material_three/
            Illustrating test_score/materials/material_two/
                Importing test_score.materials.material_two.definition
                    Abjad runtime: ... second...
                Writing test_score/materials/material_two/illustration.ly ... OK!
                Writing test_score/materials/material_two/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/materials/material_two/
        '''.replace('/', os.path.sep))
        assert not self.materials_path.joinpath(
            'material_one',
            'illustration.pdf',
            ).exists()
        assert self.materials_path.joinpath(
            'material_two',
            'illustration.pdf',
            ).exists()
        assert self.materials_path.joinpath(
            'material_three',
            'illustration.pdf',
            ).exists()

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_one_material(self, open_file_mock):
        self.create_score()
        self.create_material('test_material')
        script = commandlinetools.ManageMaterialScript()
        command = ['--illustrate', 'test_material']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
            Illustration candidates: 'test_material' ...
            Illustrating test_score/materials/test_material/
                Importing test_score.materials.test_material.definition
                    Abjad runtime: ... second...
                Writing test_score/materials/test_material/illustration.ly ... OK!
                Writing test_score/materials/test_material/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/materials/test_material/
        '''.replace('/', os.path.sep))
        self.compare_path_contents(self.materials_path, self.expected_files)
        illustration_path = self.materials_path.joinpath(
            'test_material', 'illustration.ly')
        self.compare_lilypond_contents(
            illustration_path,
            self.expected_illustration_contents,
            )



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Fermata(AbjadValueObject):
    r'''Fermata.

    ..  container:: example

        **Example 1.** A short fermata:

        ::

            >>> score = Score([Staff([Note("c'4")])])
            >>> fermata = indicatortools.Fermata(command='shortfermata')
            >>> attach(fermata, score[0][0])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4 \shortfermata
                }
            >>

    ..  container:: example

        **Example 2.** A fermata:

        ::

            >>> score = Score([Staff([Note("c'4")])])
            >>> fermata = indicatortools.Fermata()
            >>> attach(fermata, score[0][0])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4 \fermata
                }
            >>

    ..  container:: example

        **Example 3.** A long fermata:

        ::

            >>> score = Score([Staff([Note("c'4")])])
            >>> fermata = indicatortools.Fermata('longfermata')
            >>> attach(fermata, score[0][0])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4 \longfermata
                }
            >>

    ..  container:: example

        **Example 4.** A very long fermata:

        ::

            >>> score = Score([Staff([Note("c'4")])])
            >>> fermata = indicatortools.Fermata('verylongfermata')
            >>> attach(fermata, score[0][0])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4 \verylongfermata
                }
            >>

    '''

    ### CLASS VARIABLES ###

    _allowable_commands = (
        'fermata',
        'longfermata',
        'shortfermata',
        'verylongfermata',
        )

    __slots__ = (
        '_command',
        '_default_scope',
        )

    _format_slot = 'right'

    ### INITIALIZER ###

    def __init__(self, command='fermata'):
        from abjad.tools import scoretools
        assert command in self._allowable_commands, repr(command)
        self._command = command
        self._default_scope = scoretools.Score

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of fermata.

        ..  container:: example

            **Example 1.** Fermata:

            ::

                >>> str(indicatortools.Fermata())
                '\\fermata'

        ..  container:: example

            **Example 2.** Long fermata:

            ::

                >>> str(indicatortools.Fermata('longfermata'))
                '\\longfermata'

        Returns string.
        '''
        return r'\{}'.format(self.command)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return str(self)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.articulations.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC METHODS ###

    @staticmethod
    def list_allowable_commands():
        r'''Lists allowable commands:

        ..  container:: example

            **Example 1.** All allowable commands:

            ::

                >>> commands = indicatortools.Fermata.list_allowable_commands()
                >>> for command in commands:
                ...     command
                'fermata'
                'longfermata'
                'shortfermata'
                'verylongfermata'

        Returns tuple of strings.
        '''
        return Fermata._allowable_commands

    ### PUBLIC PROPERTIES ###

    @property
    def command(self):
        r'''Gets command of fermata.

        ..  container:: example

            **Example 1.** Fermata:

            ::

                >>> fermata = indicatortools.Fermata()
                >>> fermata.command
                'fermata'

        ..  container:: example

            **Example 2.** Long fermata:

            ::

                >>> fermata = indicatortools.Fermata('longfermata')
                >>> fermata.command
                'longfermata'

        Returns string.
        '''
        return self._command

    @property
    def default_scope(self):
        r'''Gets default scope of fermata.

        ..  container:: example

            **Example 1.** Fermata:

            ::

                >>> fermata = indicatortools.Fermata()
                >>> fermata.default_scope
                <class 'abjad.tools.scoretools.Score.Score'>

        ..  container:: example

            **Example 2.** Long fermata:

            ::

                >>> fermata = indicatortools.Fermata('longfermata')
                >>> fermata.default_scope
                <class 'abjad.tools.scoretools.Score.Score'>

        Fermatas are score-scoped by default.

        Returns score.
        '''
        return self._default_scope



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSet_invert_01():

    assert pitchtools.PitchClassSet([0, 1, 5]).invert() == \
        pitchtools.PitchClassSet([0, 7, 11])
    assert pitchtools.PitchClassSet([1, 2, 6]).invert() == \
        pitchtools.PitchClassSet([6, 10, 11])



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch___add___01():

    pitch = NamedPitch(12)
    named_interval = pitchtools.NamedInterval('minor', 2)

    assert pitch + named_interval == NamedPitch('df', 5)


def test_pitchtools_NamedPitch___add___02():

    pitch = NamedPitch(12)
    numbered_interval = pitchtools.NumberedInterval(1)

    assert pitch + numbered_interval == NamedPitch('df', 5)



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QEventSequence_from_tempo_scaled_leaves_01():

    staff = Staff([])

    staff.append(Note(0, (1, 4)))
    staff.append(Rest((1, 4)))
    staff.append(Rest((1, 8)))
    staff.append(Note(1, (1, 8)))
    staff.append(Note(1, (1, 8)))
    staff.append(Note(2, (1, 8)))
    staff.append(Note(2, (1, 8)))
    staff.append(Note(3, (1, 8)))
    staff.append(scoretools.Skip((1, 4)))
    staff.append(Rest((1, 4)))
    staff.append(Note(3, (1, 8)))
    staff.append(Chord([0, 1, 4], (1, 4)))

    tie = spannertools.Tie()
    attach(tie, staff[3:5])
    tie = spannertools.Tie()
    attach(tie, staff[5:7])

    tempo = Tempo((1, 4), 55)

    leaves = select(staff).by_leaf()
    q_events = quantizationtools.QEventSequence.from_tempo_scaled_leaves(
        leaves, tempo)

    assert q_events == quantizationtools.QEventSequence((
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(12000, 11)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(30000, 11),
            (NamedPitch("cs'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(42000, 11),
            (NamedPitch("d'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(54000, 11),
            (NamedPitch("ef'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(60000, 11)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(84000, 11),
            (NamedPitch("ef'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(90000, 11),
            (
                NamedPitch("c'"),
                NamedPitch("cs'"),
                NamedPitch("e'"),
            )
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(102000, 11)
            )
    ))


def test_quantizationtools_QEventSequence_from_tempo_scaled_leaves_02():

    staff = Staff([])

    staff.append(Note(0, (1, 4)))
    staff.append(Rest((1, 4)))
    staff.append(Rest((1, 8)))
    staff.append(Note(1, (1, 8)))
    staff.append(Note(1, (1, 8)))
    staff.append(Note(2, (1, 8)))
    staff.append(Note(2, (1, 8)))
    staff.append(Note(3, (1, 8)))
    staff.append(scoretools.Skip((1, 4)))
    staff.append(Rest((1, 4)))
    staff.append(Note(3, (1, 8)))
    staff.append(Chord([0, 1, 4], (1, 4)))

    tie = spannertools.Tie()
    attach(tie, staff[3:5])
    tie = spannertools.Tie()
    attach(tie, staff[5:7])

    tempo = Tempo((1, 4), 58)
    attach(tempo, staff[0], scope=Staff)
    tempo = Tempo((1, 4), 77)
    attach(tempo, staff[9], scope=Staff)

    leaves = select(staff).by_leaf()
    q_events = quantizationtools.QEventSequence.from_tempo_scaled_leaves(
        leaves)

    assert q_events == quantizationtools.QEventSequence((
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(30000, 29)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(75000, 29),
            (NamedPitch("cs'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(105000, 29),
            (NamedPitch("d'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(135000, 29),
            (NamedPitch("ef'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(150000, 29)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(15600000, 2233),
            (NamedPitch("ef'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(16470000, 2233),
            (
                NamedPitch("c'"),
                NamedPitch("cs'"),
                NamedPitch("e'"),
            )
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(18210000, 2233)
            )
    ))



# -*- coding: utf-8 -*-
import textwrap
from abjad.tools.datastructuretools.TreeNode import TreeNode


class ReSTParagraph(TreeNode):
    r'''A ReST paragraph.

    ::

        >>> paragraph = documentationtools.ReSTParagraph(
        ...     text='blah blah blah')
        >>> paragraph
        ReSTParagraph(
            text='blah blah blah',
            wrap=True
            )

    ::

        >>> print(_.rest_format)
        blah blah blah

    Handles automatic linewrapping.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(self, name=None, text='foo', wrap=True):
        TreeNode.__init__(self, name=name)
        self.text = text
        self.wrap = wrap

    ### PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        if self.wrap:
            text = ' '.join(self.text.splitlines())
            return textwrap.wrap(text)
        return [self.text]

    ### PUBLIC PROPERTIES ###

    @property
    def rest_format(self):
        r'''ReST format of ReST paragraph.

        Returns string.
        '''
        return '\n'.join(self._rest_format_contributions)

    @property
    def text(self):
        r'''Gets and sets text of ReST paragraph.

        Returns string.
        '''
        return self._text

    @text.setter
    def text(self, arg):
        assert isinstance(arg, str)
        arg = arg.strip()
        assert len(arg)
        self._text = arg

    @property
    def wrap(self):
        r'''Gets and sets wrap flag of ReST paragraph.

        Returns true or false.
        '''
        return self._wrap

    @wrap.setter
    def wrap(self, arg):
        self._wrap = bool(arg)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_leaves_from_talea_01():

    leaves = scoretools.make_leaves_from_talea([3, -3, 5, -5], 8)
    staff = Staff(leaves)

    r'''
    \new Staff {
      c'4.
      r4.
      c'2 ~
      c'8
      r2
      r8
    }
    '''

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4.
            r4.
            c'2 ~
            c'8
            r2
            r8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class RegistrationList(TypedList):
    '''Registration list.

    ..  container:: example

        Two registrations:
        
        ::

            >>> registration_1 = pitchtools.Registration(
            ...     [('[A0, C4)', 15), ('[C4, C8)', 27)]
            ...     )
            >>> registration_2 = pitchtools.Registration(
            ...     [('[A0, C8]', -18)]
            ...     )
            >>> registrations = pitchtools.RegistrationList(
            ...     [registration_1, registration_2]
            ...     )

        ::

            >>> f(registrations)
            pitchtools.RegistrationList(
                [
                    pitchtools.Registration(
                        [
                            pitchtools.RegistrationComponent(
                                source_pitch_range=pitchtools.PitchRange(
                                    range_string='[A0, C4)',
                                    ),
                                target_octave_start_pitch=pitchtools.NumberedPitch(15),
                                ),
                            pitchtools.RegistrationComponent(
                                source_pitch_range=pitchtools.PitchRange(
                                    range_string='[C4, C8)',
                                    ),
                                target_octave_start_pitch=pitchtools.NumberedPitch(27),
                                ),
                            ]
                        ),
                    pitchtools.Registration(
                        [
                            pitchtools.RegistrationComponent(
                                source_pitch_range=pitchtools.PitchRange(
                                    range_string='[A0, C8]',
                                    ),
                                target_octave_start_pitch=pitchtools.NumberedPitch(-18),
                                ),
                            ]
                        ),
                    ]
                )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        from abjad.tools import pitchtools
        return pitchtools.Registration



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QGridContainer___eq___01():

    a = quantizationtools.QGridContainer(preprolated_duration=1, children=[])
    b = quantizationtools.QGridContainer(preprolated_duration=1, children=[])

    assert format(a) == format(b)
    assert a != b


def test_quantizationtools_QGridContainer___eq___02():

    a = quantizationtools.QGridContainer(preprolated_duration=1, children=[
        quantizationtools.QGridLeaf(preprolated_duration=1)
        ])
    b = quantizationtools.QGridContainer(preprolated_duration=1, children=[
        quantizationtools.QGridLeaf(preprolated_duration=1)
        ])

    assert format(a) == format(b)
    assert a != b


def test_quantizationtools_QGridContainer___eq___03():

    a = quantizationtools.QGridContainer(preprolated_duration=1, children=[])
    b = quantizationtools.QGridContainer(preprolated_duration=2, children=[])
    c = quantizationtools.QGridContainer(preprolated_duration=1, children=[
        quantizationtools.QGridLeaf(preprolated_duration=1)
        ])
    d = quantizationtools.QGridContainer(preprolated_duration=2, children=[
        quantizationtools.QGridLeaf(preprolated_duration=1)
        ])
    e = quantizationtools.QGridContainer(preprolated_duration=2, children=[
        quantizationtools.QGridLeaf(2)
        ])

    assert a != b
    assert a != c
    assert a != d
    assert a != e
    assert b != c
    assert b != d
    assert b != e
    assert c != d
    assert c != e
    assert d != e



# -*- coding: utf-8 -*-
import abc
from abjad.tools.abctools import AbjadObject


class Heuristic(AbjadObject):
    r'''Abstract base class from which concrete ``Heuristic``
    subclases inherit.

    ``Heuristics`` rank ``QGrids`` according to the criteria they
    encapsulate.

    They provide the means by which the quantizer selects a single ``QGrid``
    from all computed ``QGrids`` for any given ``QTargetBeat`` to
    represent that beat.
    '''

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self, q_target_beats):
        r'''Calls heuristic.

        Returns none.
        '''
        from abjad.tools import quantizationtools
        assert len(q_target_beats)
        assert all(isinstance(x, quantizationtools.QTargetBeat)
            for x in q_target_beats)
        return self._process(q_target_beats)

    ### PRIVATE METHODS ###

    @abc.abstractmethod
    def _process(self, q_target_beats):
        raise NotImplementedError



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__leaves__Note_01():

    target = Note(0, 1)
    parser = LilyPondParser()
    result = parser('{ %s }' % format(target))
    assert format(target) == format(result[0]) and target is not result[0]



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.instrumenttools.Instrument import Instrument


class Accordion(Instrument):
    r'''An accordion.

    ::

        >>> staff_group = StaffGroup()
        >>> staff_group.context_name = 'PianoStaff'
        >>> staff_group.append(Staff("c'4 d'4 e'4 f'4"))
        >>> staff_group.append(Staff("c'2 b2"))
        >>> accordion = instrumenttools.Accordion()
        >>> attach(accordion, staff_group)
        >>> attach(Clef(name='bass'), staff_group[1])
        >>> show(staff_group) # doctest: +SKIP

    ..  doctest::

        >>> f(staff_group)
        \new PianoStaff <<
            \set PianoStaff.instrumentName = \markup { Accordion }
            \set PianoStaff.shortInstrumentName = \markup { Acc. }
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }
            \new Staff {
                \clef "bass"
                c'2
                b2
            }
        >>

    The accordion targets the piano staff context by default.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='accordion',
        short_instrument_name='acc.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=('treble', 'bass'),
        pitch_range='[E1, C8]',
        sounding_pitch_of_written_middle_c=None,
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._default_scope = 'PianoStaff'
        self._performer_names.extend([
            'keyboardist',
            'accordionist',
            ])
        self._is_primary_instrument = True

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats accordion.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            ::

                >>> accordion = instrumenttools.Accordion()

                >>> print(format(accordion))
                instrumenttools.Accordion(
                    instrument_name='accordion',
                    short_instrument_name='acc.',
                    instrument_name_markup=markuptools.Markup(
                        contents=['Accordion'],
                        ),
                    short_instrument_name_markup=markuptools.Markup(
                        contents=['Acc.'],
                        ),
                    allowable_clefs=instrumenttools.ClefList(
                        [
                            indicatortools.Clef(
                                name='treble',
                                ),
                            indicatortools.Clef(
                                name='bass',
                                ),
                            ]
                        ),
                    pitch_range=pitchtools.PitchRange(
                        range_string='[E1, C8]',
                        ),
                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                    )

        Returns string.
        '''
        superclass = super(Accordion, self)
        return superclass.__format__(format_specification=format_specification)

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets accordion's allowable clefs.

        ..  container:: example

            ::

                >>> accordion.allowable_clefs
                ClefList([Clef(name='treble'), Clef(name='bass')])

            ::

                >>> show(accordion.allowable_clefs) # doctest: +SKIP

        Returns clef list.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets accordion's name.

        ..  container:: example

            ::

                >>> accordion.instrument_name
                'accordion'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets accordion's instrument name markup.

        ..  container:: example

            ::

                >>> accordion.instrument_name_markup
                Markup(contents=['Accordion'])

            ::

                >>> show(accordion.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets accordion's range.

        ..  container:: example

            ::

                >>> accordion.pitch_range
                PitchRange(range_string='[E1, C8]')

            ::

                >>> show(accordion.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets accordion's short instrument name.

        ..  container:: example

            ::

                >>> accordion.short_instrument_name
                'acc.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets accordion's short instrument name markup.

        ..  container:: example

            ::

                >>> accordion.short_instrument_name_markup
                Markup(contents=['Acc.'])

            ::

                >>> show(accordion.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of accordion's written middle C.

        ..  container:: example

            ::

                >>> accordion.sounding_pitch_of_written_middle_c
                NamedPitch("c'")

            ::

                >>> show(accordion.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)



# -*- coding: utf-8 -*-


def is_integer_equivalent_singleton(argument):
    r'''Is true when `argument` is a singleton of integer-equivalent items.
    Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_integer_equivalent_singleton((2.0,))
            True

        ::

            >>> mathtools.is_integer_equivalent_singleton((2.5,))
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    return (
        isinstance(argument, tuple) and
        len(argument) == 1 and
        all(mathtools.is_integer_equivalent(_) for _ in argument)
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QEventSequence_from_millisecond_pitch_pairs_01():

    durations = [100, 200, 100, 300, 350, 400, 600]
    pitches = [0, None, None, [1, 4], None, 5, 7]
    pairs = tuple(zip(durations, pitches))

    q_events = quantizationtools.QEventSequence.from_millisecond_pitch_pairs(
        pairs)

    assert q_events == quantizationtools.QEventSequence((
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(100, 1)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(400, 1),
            (
                NamedPitch("cs'"),
                NamedPitch("e'")
            )
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(700, 1)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1050, 1),
            (NamedPitch("f'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1450, 1),
            (NamedPitch("g'"),)
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(2050, 1),
            )
    ))



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordExtent_name_01():

    assert tonalanalysistools.ChordExtent(5).name == 'triad'
    assert tonalanalysistools.ChordExtent(7).name == 'seventh'
    assert tonalanalysistools.ChordExtent(9).name == 'ninth'



# -*- coding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import schemetools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import override


def make_dynamic_spanner_below_with_nib_at_right(dynamic_text):
    r'''Makes dynamic spanner below with nib at right.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.make_dynamic_spanner_below_with_nib_at_right('mp')
            >>> attach(spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \override TextSpanner.bound-details.left.text = \markup { \dynamic { mp } }
                \override TextSpanner.bound-details.right-broken.text = ##f
                \override TextSpanner.bound-details.right.text = \markup {
                    \draw-line
                        #'(0 . 1)
                    }
                \override TextSpanner.dash-fraction = #1
                \override TextSpanner.direction = #down
                c'8 \startTextSpan
                d'8
                e'8
                f'8 \stopTextSpan
                \revert TextSpanner.bound-details
                \revert TextSpanner.dash-fraction
                \revert TextSpanner.direction
            }

    Returns text spanner.
    '''
    from abjad.tools import spannertools

    text_spanner = spannertools.TextSpanner()

    string = r'\dynamic {{ {} }}'.format(dynamic_text)
    left_markup = markuptools.Markup(string)

    pair = schemetools.SchemePair(0, 1)
    markup_command = markuptools.MarkupCommand('draw-line', pair)
    right_markup = markuptools.Markup(markup_command)

    grob = override(text_spanner).text_spanner
    grob.bound_details__left__text = left_markup
    grob.bound_details__right__text = right_markup
    grob.bound_details__right_broken__text = False
    grob.dash_fraction = 1
    grob.direction = Down

    return text_spanner



# -*- coding: utf-8 -*-
import copy


def make_lilypond_file(
    selections,
    divisions=None,
    attach_lilypond_voice_commands=None,
    implicit_scaling=None,
    pitched_staff=None,
    simultaneous_selections=None,
    time_signatures=None,
    ):
    r'''Makes LilyPond file.

    ..  container:: example

        Makes rhythmic staff:

        ::

            >>> maker = rhythmmakertools.EvenRunRhythmMaker(exponent=1)
            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> selections = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> score = lilypond_file[Score]
            >>> f(score)
            \new Score <<
                \new TimeSignatureContext {
                    {
                        \time 3/4
                        s1 * 3/4
                    }
                    {
                        \time 4/8
                        s1 * 1/2
                    }
                    {
                        \time 1/4
                        s1 * 1/4
                    }
                }
                \new RhythmicStaff {
                    {
                        \time 3/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 1/4
                        {
                            c'8 [
                            c'8 ]
                        }
                    }
                }
            >>

    ..  container:: example

        Set time signatures explicitly:

        ::

            >>> maker = rhythmmakertools.EvenRunRhythmMaker(exponent=1)
            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> selections = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     [(6, 8), (4, 8), (2, 8)],
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> score = lilypond_file[Score]
            >>> f(score)
            \new Score <<
                \new TimeSignatureContext {
                    {
                        \time 6/8
                        s1 * 3/4
                    }
                    {
                        \time 4/8
                        s1 * 1/2
                    }
                    {
                        \time 2/8
                        s1 * 1/4
                    }
                }
                \new RhythmicStaff {
                    {
                        \time 6/8
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 2/8
                        {
                            c'8 [
                            c'8 ]
                        }
                    }
                }
            >>

    ..  container:: example

        Makes pitched staff:

        ::

            >>> maker = rhythmmakertools.EvenRunRhythmMaker(exponent=1)
            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> selections = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     pitched_staff=True,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> score = lilypond_file[Score]
            >>> f(score)
            \new Score <<
                \new TimeSignatureContext {
                    {
                        \time 3/4
                        s1 * 3/4
                    }
                    {
                        \time 4/8
                        s1 * 1/2
                    }
                    {
                        \time 1/4
                        s1 * 1/4
                    }
                }
                \new Staff {
                    {
                        \time 3/4
                        {
                            c'8 [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8 ]
                        }
                    }
                    {
                        \time 4/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \time 1/4
                        {
                            c'8 [
                            c'8 ]
                        }
                    }
                }
            >>

    ..  container:: example

        Makes simultaneous voices:

        ::

            >>> maker_1 = rhythmmakertools.EvenRunRhythmMaker(exponent=1)
            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> selection_1 = select(maker_1(divisions))
            >>> for note in iterate(selection_1).by_class(Note):
            ...     note.written_pitch = NamedPitch("e'")
            ...
            >>> maker_2 = rhythmmakertools.EvenRunRhythmMaker(exponent=2)
            >>> selection_2 = select(maker_2(divisions))
            >>> selections = {
            ...     'Voice 1': selection_1,
            ...     'Voice 2': selection_2,
            ... }
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> voice_1 = lilypond_file['Voice 1']
            >>> attach(LilyPondCommand('voiceOne'), voice_1)
            >>> voice_2 = lilypond_file['Voice 2']
            >>> attach(LilyPondCommand('voiceTwo'), voice_2)
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Score])
            \new Score <<
                \new TimeSignatureContext {
                    {
                        \time 3/4
                        s1 * 3/4
                    }
                    {
                        \time 4/8
                        s1 * 1/2
                    }
                    {
                        \time 1/4
                        s1 * 1/4
                    }
                }
                \new Staff <<
                    \context Voice = "Voice 1" {
                        \voiceOne
                        {
                            e'8 [
                            e'8
                            e'8
                            e'8
                            e'8
                            e'8 ]
                        }
                        {
                            e'16 [
                            e'16
                            e'16
                            e'16
                            e'16
                            e'16
                            e'16
                            e'16 ]
                        }
                        {
                            e'8 [
                            e'8 ]
                        }
                    }
                    \context Voice = "Voice 2" {
                        \voiceTwo
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                        {
                            c'32 [
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32 ]
                        }
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                >>
            >>

    Used in rhythm-maker docs.

    Returns LilyPond file.
    '''
    import abjad
    if isinstance(selections, list):
        for selection in selections:
            if not isinstance(selection, abjad.Selection):
                message = 'must be selection: {!r}.'
                message = message.format(selection)
                raise TypeError(message)
    elif isinstance(selections, dict):
        for selection in selections.values():
            if not isinstance(selection, abjad.Selection):
                message = 'must be selection: {!r}.'
                message = message.format(selection)
                raise TypeError(message)
    else:
        message = 'must be list or dictionary: {!r}.'
        message = message.format(selections)
        raise TypeError(message)
    score = abjad.Score()
    package = abjad.lilypondfiletools
    lilypond_file = package.make_floating_time_signature_lilypond_file(score)
    if pitched_staff is None:
        for note in abjad.iterate(selections).by_class(
            abjad.Note,
            with_grace_notes=True,
            ):
            if note.written_pitch != abjad.NamedPitch("c'"):
                pitched_staff = True
                break
    if isinstance(selections, list):
        if divisions is None:
            duration = sum([_.get_duration() for _ in selections])
            divisions = [duration]
        time_signatures = time_signatures or divisions
        measures = abjad.scoretools.make_spacer_skip_measures(
            time_signatures,
            implicit_scaling=implicit_scaling,
            )
        if pitched_staff:
            staff = abjad.Staff(measures)
        else:
            staff = abjad.Staff(measures, context_name='RhythmicStaff')
        selections = abjad.Sequence(selections).flatten()
        selections_ = copy.deepcopy(selections)
        try:
            measures = abjad.mutate(staff).replace_measure_contents(selections)
        except StopIteration:
            if pitched_staff:
                staff = abjad.Staff(selections_)
            else:
                staff = abjad.Staff(selections_, context_name='RhythmicStaff')
    elif isinstance(selections, dict):
        voices = []
        for voice_name in sorted(selections):
            selections_ = selections[voice_name]
            selections_ = abjad.Sequence(selections_).flatten()
            selections_ = copy.deepcopy(selections_)
            voice = abjad.Voice(selections_, name=voice_name)
            if attach_lilypond_voice_commands:
                voice_name_to_command_string = {
                    'Voice 1': 'voiceOne',
                    'Voice 2': 'voiceTwo',
                    'Voice 3': 'voiceThree',
                    'Voice 4': 'voiceFour',
                    }
                command_string = voice_name_to_command_string.get(voice_name)
                if command_string:
                    command = abjad.LilyPondCommand(command_string)
                    abjad.attach(command, voice)
            voices.append(voice)
        staff = abjad.Staff(voices, is_simultaneous=True)
        if divisions is None:
            duration = abjad.inspect_(staff).get_duration()
            divisions = [duration]
    else:
        message = 'must be list or dictionary of selections: {!r}.'
        message = message.format(selections)
        raise TypeError(message)
    score.append(staff)
    assert isinstance(divisions, (tuple, list)), repr(divisions)
    time_signatures = time_signatures or divisions
    context = abjad.scoretools.Context(context_name='TimeSignatureContext')
    measures = abjad.scoretools.make_spacer_skip_measures(
        time_signatures,
        implicit_scaling=implicit_scaling,
        )
    context.extend(measures)
    score.insert(0, context)
    return lilypond_file



# -*- coding: utf-8 -*-
import functools
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


@functools.total_ordering
class OrdinalConstant(AbjadValueObject):
    r'''An ordinal constant.

    ..  container:: example

        Initializes with `dimension`, `value` and `representation`:

        ::

            >>> Left = datastructuretools.OrdinalConstant('x', -1, 'Left')
            >>> Left
            Left

        ::

            >>> Right = datastructuretools.OrdinalConstant('x', 1, 'Right')
            >>> Right
            Right

        ::

            >>> Left < Right
            True

    ..  container:: example

        Comparing like-dimensioned ordinal constants is allowed:

        ::

            >>> Up = datastructuretools.OrdinalConstant('y', 1, 'Up')
            >>> Up
            Up

        ::

            >>> Down = datastructuretools.OrdinalConstant('y', -1, 'Down')
            >>> Down
            Down

        ::

            >>> Down < Up
            True

    ..  container:: example

        Comparing differently dimensioned ordinal constants raises an exception:

        ::

            >>> import pytest

        ::

            >>> bool(pytest.raises(Exception, 'Left < Up'))
            True

    Abjad adds the following constants to Python's built-in namespace
    when Abjad is first imported:

    * ``Left``
    * ``Right``
    * ``Center``
    * ``Up``
    * ``Down``
    * ``Less``
    * ``More``
    * ``Exact``

    These eight objects can be used as constant values supplied to keywords.

    This behavior is similar to built-in ``True``, ``False`` and ``None``.

    ..  note:: Use ``==`` to compare Abjad ordinal constants.
        Do not use ``is``.

    Ordinal constants are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_dimension',
        '_representation',
        '_value',
        )

    ### CONSTRUCTOR ###

    def __new__(
        class_,
        dimension=None,
        value=0,
        representation=None,
        ):
        dimension = dimension or ''
        representation = representation or ''
        assert isinstance(dimension, str), repr(dimension)
        assert isinstance(representation, str), repr(representation)
        self = object.__new__(class_)
        self._dimension = dimension
        self._value = value
        self._representation = representation
        return self

    ### SPECIAL METHODS ###

    def __lt__(self, argument):
        r'''Is true when `argument` is an ordinal with value greater than that of
        this ordinal constant. Otherwise false.

        Returns true or false.
        '''
        self._check_comparator(argument)
        return self._value < argument._value

    ### PRIVATE METHODS ###

    # can only compare like-dimensioned ordinal constants
    def _check_comparator(self, argument):
        if not isinstance(argument, type(self)) or \
            self._dimension != argument._dimension:
            message = 'can only compare like-dimensioned ordinal constants.'
            raise Exception(message)

    def _get_format_specification(self):
        storage_format_text = repr_text = self._representation or None
        return systemtools.FormatSpecification(
            client=self,
            repr_text=repr_text,
            storage_format_text=storage_format_text,
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Hairpin_start_dynamic_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    hairpin = Hairpin(descriptor='p < f')
    attach(hairpin, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \< \p
            d'8
            e'8
            f'8 \f
        }
        '''
        )

    assert hairpin.start_dynamic == Dynamic('p')


def test_spannertools_Hairpin_start_dynamic_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    hairpin = Hairpin(descriptor='mf < f')
    attach(hairpin, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \< \mf
            d'8
            e'8
            f'8 \f
        }
        '''
        )

    assert hairpin.start_dynamic == Dynamic('mf')



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_RomanNumeral___init___01():
    r'''Initializes roman numeral from empty input.
    '''

    roman_numeral = tonalanalysistools.RomanNumeral()

    assert repr(roman_numeral) == 'IMajorTriadInRootPosition'



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Container_is_simultaneous_01():
    r'''Is true when container encloses contents in LilyPond << >> brackets,
    otherwise False.
    '''

    assert not Container().is_simultaneous
    assert not scoretools.FixedDurationTuplet().is_simultaneous
    assert not Tuplet().is_simultaneous
    assert not Measure().is_simultaneous
    assert Score().is_simultaneous
    assert not Container().is_simultaneous
    assert not Staff().is_simultaneous
    assert scoretools.StaffGroup().is_simultaneous
    assert not Voice().is_simultaneous


def test_scoretools_Container_is_simultaneous_02():
    r'''Is true when container encloses contents in LilyPond << >> brackets,
    otherwise False.
    '''

    container = Container([])
    container.is_simultaneous = True
    assert container.is_simultaneous


def test_scoretools_Container_is_simultaneous_03():
    r'''Container 'simultaneous' is settable.
    '''

    container = Container([])
    assert not container.is_simultaneous

    container.is_simultaneous = True
    assert container.is_simultaneous


def test_scoretools_Container_is_simultaneous_04():
    r'''A simultaneous container can hold Contexts.
    '''

    container = Container([Voice("c'8 cs'8"), Voice("d'8 ef'8")])
    container.is_simultaneous = True

    assert format(container) == stringtools.normalize(
        r'''
        <<
            \new Voice {
                c'8
                cs'8
            }
            \new Voice {
                d'8
                ef'8
            }
        >>
        '''
        )


def test_scoretools_Container_is_simultaneous_05():
    r'''Simultaneous containers must contain only contexts.
    '''

    container = Container("c'8 c'8 c'8 c'8")
    pytest.raises(Exception, 'container.is_simultaneous = True')


def test_scoretools_Container_is_simultaneous_06():
    r'''Simultaneous containers must contain only Contexts.
    '''

    container = Container(2 * Container("c'8 c'8 c'8 c'8"))
    pytest.raises(Exception, 'container.is_simultaneous = True')



# -*- coding: utf-8 -*-
from abjad.tools.pitchtools.Vector import Vector


class PitchClassVector(Vector):
    '''Pitch-class vector.

    ..  container:: example

        Pitch-class vector:

        ::

            >>> vector = pitchtools.PitchClassVector(
            ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
            ...     item_class=pitchtools.NumberedPitchClass,
            ...     )

        ::

            >>> items = sorted(vector.items())
            >>> for pitch_class, count in items:
            ...     print(pitch_class, count)
            0 1
            1 1
            2 1
            3 1
            4 2
            6 1
            7 1
            9 2
            10 1

    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Gets format of pitch-class vector.

        ..  container:: example

            ::

                >>> vector = pitchtools.PitchClassVector(
                ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
                ...     item_class=pitchtools.NumberedPitchClass,
                ...     )

            ::

                >>> f(vector)
                pitchtools.PitchClassVector(
                    {
                        pitchtools.NumberedPitchClass(0): 1,
                        pitchtools.NumberedPitchClass(1): 1,
                        pitchtools.NumberedPitchClass(2): 1,
                        pitchtools.NumberedPitchClass(3): 1,
                        pitchtools.NumberedPitchClass(4): 2,
                        pitchtools.NumberedPitchClass(6): 1,
                        pitchtools.NumberedPitchClass(7): 1,
                        pitchtools.NumberedPitchClass(9): 2,
                        pitchtools.NumberedPitchClass(10): 1,
                        },
                    item_class=pitchtools.NumberedPitchClass,
                    )

        Returns string.
        '''
        superclass = super(PitchClassVector, self)
        return superclass.__format__(format_specification=format_specification)

    def __repr__(self):
        r'''Gets interpreter representation of pitch-class vector.

        ..  container:: example

            Gets interpreter representation of pitch-class vector:

            ::

                >>> vector = pitchtools.PitchClassVector(
                ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
                ...     item_class=pitchtools.NumberedPitchClass,
                ...     )

            ::

                >>> vector
                PitchClassVector({0: 1, 1: 1, 2: 1, 3: 1, 4: 2, 6: 1, 7: 1, 9: 2, 10: 1}, item_class=NumberedPitchClass)

        ..  container:: example

            Initializes from interpreter representation of pitch-class vector:


                >>> pitchtools.PitchClassVector(vector)
                PitchClassVector({0: 1, 1: 1, 2: 1, 3: 1, 4: 2, 6: 1, 7: 1, 9: 2, 10: 1}, item_class=NumberedPitchClass)

        Returns string.
        '''
        superclass = super(PitchClassVector, self)
        return superclass.__repr__()

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.PitchClass

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        r'''Makes pitch-class vector from `selection`.

        Returns pitch-class vector.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return class_(
            pitch_segment,
            item_class=item_class,
            )



# -*- coding: utf-8 -*-
import copy
import math
import re
from abjad import Fraction
from abjad.tools import mathtools
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.topleveltools.override import override
from abjad.tools.topleveltools.set_ import set_


class Duration(AbjadObject, Fraction):
    r'''Duration.

    ..  container:: example

        **Example 1.** Initializes from integer numerator:

        ::

            >>> Duration(3)
            Duration(3, 1)

    ..  container:: example

        **Example 2.** Initializes from integer numerator and denominator:

        ::

            >>> Duration(3, 16)
            Duration(3, 16)

    ..  container:: example

        **Example 3.** Initializes from integer-equivalent numeric numerator:

        ::

            >>> Duration(3.0)
            Duration(3, 1)

    ..  container:: example

        **Example 4.** Initializes from integer-equivalent numeric numerator
        and denominator:

        ::

            >>> Duration(3.0, 16)
            Duration(3, 16)

    ..  container:: example

        **Example 5.** Initializes from integer-equivalent singleton:

        ::

            >>> Duration((3,))
            Duration(3, 1)

    ..  container:: example

        **Example 6.** Initializes from integer-equivalent pair:

        ::

            >>> Duration((3, 16))
            Duration(3, 16)

    ..  container:: example

        **Example 7.** Initializes from other duration:

        ::

            >>> Duration(Duration(3, 16))
            Duration(3, 16)

    ..  container:: example

        **Example 8.** Intializes from fraction:

        ::

            >>> Duration(Fraction(3, 16))
            Duration(3, 16)

    ..  container:: example

        **Example 9.** Initializes from solidus string:

        ::

            >>> Duration('3/16')
            Duration(3, 16)

    ..  container:: example

        **Example 10.** Initializes from nonreduced fraction:

        ::

            >>> Duration(mathtools.NonreducedFraction(6, 32))
            Duration(3, 16)

    ..  container:: example

        **Example 11.** Durations inherit from built-in fraction:

        ::

            >>> isinstance(Duration(3, 16), Fraction)
            True

    ..  container:: example

        **Example 12.** Durations are numeric:

        ::

            >>> import numbers

        ::

            >>> isinstance(Duration(3, 16), numbers.Number)
            True

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### CONSTRUCTOR ###

    def __new__(class_, *arguments):
        if len(arguments) == 1:
            arg = arguments[0]
            if type(arg) is class_:
                return arg
            if isinstance(arg, mathtools.NonreducedFraction):
                return Fraction.__new__(class_, *arg.pair)
            try:
                return Fraction.__new__(class_, *arg)
            except (AttributeError, TypeError):
                pass
            try:
                return Fraction.__new__(class_, arg)
            except (AttributeError, TypeError):
                pass
            if mathtools.is_fraction_equivalent_pair(arg):
                return Fraction.__new__(
                    class_, int(arg[0]), int(arg[1]))
            if hasattr(arg, 'duration'):
                return Fraction.__new__(class_, arg.duration)
            if isinstance(arg, str) and '/' not in arg:
                result = Duration._initialize_from_lilypond_duration_string(
                    arg)
                return Fraction.__new__(class_, result)
            if mathtools.is_integer_equivalent_singleton(arg):
                return Fraction.__new__(class_, int(arg[0]))
        else:
            try:
                return Fraction.__new__(class_, *arguments)
            except TypeError:
                pass
            if mathtools.all_are_integer_equivalent_numbers(arguments):
                return Fraction.__new__(
                    class_,
                    *[int(x) for x in arguments]
                    )
        message = 'can not construct duration: {!r}.'
        message = message.format(arguments)
        raise ValueError(message)

    ### SPECIAL METHODS ###

    def __abs__(self, *arguments):
        r'''Gets absolute value of duration.

        Returns nonnegative duration.
        '''
        return type(self)(Fraction.__abs__(self, *arguments))

    def __add__(self, *arguments):
        r'''Adds duration to `arguments`.

        ..  container:: example

            **Example 1.** Returns duration when `arguments` is a duration:

            ::

                >>> duration_1 = Duration(1, 2)
                >>> duration_2 = Duration(3, 2)
                >>> duration_1 + duration_2
                Duration(2, 1)

        ..  container:: example

            **Example 2.** Returns nonreduced fraction when `arguments` is a
            nonreduced fraction:

            ::

                >>> duration = Duration(1, 2)
                >>> nonreduced_fraction = mathtools.NonreducedFraction(3, 6)
                >>> duration + nonreduced_fraction
                NonreducedFraction(6, 6)

        Returns duration.
        '''
        if (
            len(arguments) == 1 and
            isinstance(arguments[0], mathtools.NonreducedFraction)
            ):
            result = arguments[0].__radd__(self)
        else:
            result = type(self)(Fraction.__add__(self, *arguments))
        return result

    def __div__(self, *arguments):
        r'''Divides duration by `arguments`.

        Returns multiplier.
        '''
        from abjad.tools import durationtools
        if len(arguments) == 1 and isinstance(arguments[0], type(self)):
            fraction = Fraction.__truediv__(self, *arguments)
            result = durationtools.Multiplier(fraction)
        elif len(arguments) == 1 and isinstance(
            arguments[0], mathtools.NonreducedFraction):
            result = arguments[0].__rdiv__(self)
        else:
            result = type(self)(Fraction.__truediv__(self, *arguments))
        return result

    def __divmod__(self, *arguments):
        r'''Equals the pair (duration // `arguments`, duration % `arguments`).

        Returns pair.
        '''
        truncated, residue = Fraction.__divmod__(self, *arguments)
        truncated = type(self)(truncated)
        residue = type(self)(residue)
        return truncated, residue

    def __eq__(self, arg):
        r'''Is true when duration equals `arg`.
        Otherwise false.

        Returns true or false.
        '''
        return Fraction.__eq__(self, arg)

    def __format__(self, format_specification=''):
        r'''Formats duration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __ge__(self, arg):
        r'''Is true when duration is greater than or equal to `arg`.
        Otherwise false.

        Returns true or false.
        '''
        return Fraction.__ge__(self, arg)

    def __gt__(self, arg):
        r'''Is true when duration is greater than `arg`.
        Otherwise false.

        Returns true or false.
        '''
        return Fraction.__gt__(self, arg)

    def __hash__(self):
        r'''Hashes duration.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Duration, self).__hash__()

    def __le__(self, arg):
        r'''Is true when duration is less than or equal to `arg`.
        Otherwise false.

        Returns true or false.
        '''
        return Fraction.__le__(self, arg)

    def __lt__(self, arg):
        r'''Is true when duration is less than `arg`.
        Otherwise false.

        Returns true or false.
        '''
        return Fraction.__lt__(self, arg)

    def __mod__(self, *arguments):
        r'''Modulus operator applied to duration.

        Returns duration.
        '''
        return type(self)(Fraction.__mod__(self, *arguments))

    def __mul__(self, *arguments):
        r'''Duration multiplied by `arguments`.

        ..  container:: example

            **Example 1.** Returns a new duration when `arguments` is a duration:

            ::

                >>> duration_1 = Duration(1, 2)
                >>> duration_2 = Duration(3, 2)
                >>> duration_1 * duration_2
                Duration(3, 4)

        ..  container:: example

            **Example 2.** Returns nonreduced fraction when `arguments` is a
            nonreduced fraction:

            ::

                >>> duration = Duration(1, 2)
                >>> nonreduced_fraction = mathtools.NonreducedFraction(3, 6)
                >>> duration * nonreduced_fraction
                NonreducedFraction(3, 12)

        Returns duration or nonreduced fraction.
        '''
        if (
            len(arguments) == 1 and
            isinstance(arguments[0], mathtools.NonreducedFraction)
            ):
            result = arguments[0].__rmul__(self)
        else:
            result = type(self)(Fraction.__mul__(self, *arguments))
        return result

    def __ne__(self, arg):
        r'''Is true when duration does not equal `arg`.
        Otherwise false.

        Returns true or false.
        '''
        return Fraction.__ne__(self, arg)

    def __neg__(self, *arguments):
        r'''Negates duration.

        Returns new duration.
        '''
        return type(self)(Fraction.__neg__(self, *arguments))

    def __pos__(self, *arguments):
        r'''Get positive duration.

        Returns new duration.
        '''
        return type(self)(Fraction.__pos__(self, *arguments))

    def __pow__(self, *arguments):
        r'''Raises duration to `arguments` power.

        Returns new duration.
        '''
        return type(self)(Fraction.__pow__(self, *arguments))

    def __radd__(self, *arguments):
        r'''Adds `arguments` to duration.

        Returns new duration.
        '''
        return type(self)(Fraction.__radd__(self, *arguments))

    def __rdiv__(self, *arguments):
        r'''Divides `arguments` by duration.

        Returns new duration.
        '''
        return type(self)(Fraction.__rdiv__(self, *arguments))

    def __rdivmod__(self, *arguments):
        r'''Documentation required.
        '''
        return type(self)(Fraction.__rdivmod__(self, *arguments))

    def __reduce__(self):
        r'''Documentation required.
        '''
        return type(self), (self.numerator, self.denominator)

    def __reduce_ex__(self, protocol):
        r'''Documentation required.
        '''
        return type(self), (self.numerator, self.denominator)

    def __rmod__(self, *arguments):
        r'''Documentation required.
        '''
        return type(self)(Fraction.__rmod__(self, *arguments))

    def __rmul__(self, *arguments):
        r'''Multiplies `arguments` by duration.

        Returns new duration.
        '''
        return type(self)(Fraction.__rmul__(self, *arguments))

    def __rpow__(self, *arguments):
        r'''Raises `arguments` to the power of duration.

        Returns new duration.
        '''
        return type(self)(Fraction.__rpow__(self, *arguments))

    def __rsub__(self, *arguments):
        r'''Subtracts duration from `arguments`.

        Returns new duration.
        '''
        return type(self)(Fraction.__rsub__(self, *arguments))

    def __rtruediv__(self, *arguments):
        r'''Documentation required.

        Returns new duration.
        '''
        return type(self)(Fraction.__rtruediv__(self, *arguments))

    def __sub__(self, *arguments):
        r'''Subtracts `arguments` from duration.

        Returns new duration.
        '''
        if (
            len(arguments) == 1 and
            isinstance(arguments[0], mathtools.NonreducedFraction)
            ):
            return arguments[0].__rsub__(self)
        else:
            return type(self)(Fraction.__sub__(self, *arguments))

    def __truediv__(self, *arguments):
        r'''Documentation required.
        '''
        return self.__div__(*arguments)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=[
                self.numerator,
                self.denominator,
                ],
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            )

    @staticmethod
    def _group_nonreduced_fractions_by_implied_prolation(durations):
        durations = [
            mathtools.NonreducedFraction(duration)
            for duration in durations
            ]
        assert 0 < len(durations)
        group = [durations[0]]
        result = [group]
        for d in durations[1:]:
            d_f = set(mathtools.factors(d.denominator))
            d_f.discard(2)
            gd_f = set(mathtools.factors(group[0].denominator))
            gd_f.discard(2)
            if d_f == gd_f:
                group.append(d)
            else:
                group = [d]
                result.append(group)
        return result

    @staticmethod
    def _initialize_from_lilypond_duration_string(duration_string):
        numeric_body_strings = [str(2 ** n) for n in range(8)]
        other_body_strings = [r'\\breve', r'\\longa', r'\\maxima']
        body_strings = numeric_body_strings + other_body_strings
        body_strings = '|'.join(body_strings)
        pattern = r'^(%s)(\.*)$' % body_strings
        match = re.match(pattern, duration_string)
        if match is None:
            message = 'incorrect duration string format: {!r}.'
            message = message.format(duration_string)
            raise TypeError(message)
        body_string, dots_string = match.groups()
        try:
            body_denominator = int(body_string)
            body_duration = Fraction(1, body_denominator)
        except ValueError:
            if body_string == r'\breve':
                body_duration = Fraction(2)
            elif body_string == r'\longa':
                body_duration = Fraction(4)
            elif body_string == r'\maxima':
                body_duration = Fraction(8)
            else:
                message = 'unknown body string: {!r}.'
                message = message.format(body_string)
                raise ValueError(message)
        rational = body_duration
        for n in range(len(dots_string)):
            exponent = n + 1
            denominator = 2 ** exponent
            multiplier = Fraction(1, denominator)
            addend = multiplier * body_duration
            rational += addend
        return rational

    @staticmethod
    def _make_markup_score_block(selection):
        from abjad.tools import lilypondfiletools
        from abjad.tools import schemetools
        from abjad.tools import scoretools
        selection = copy.deepcopy(selection)
        staff = scoretools.Staff(selection)
        staff.context_name = 'RhythmicStaff'
        staff.remove_commands.append('Time_signature_engraver')
        staff.remove_commands.append('Staff_symbol_engraver')
        override(staff).stem.direction = Up
        #override(staff).stem.length = 4
        override(staff).stem.length = 5
        override(staff).tuplet_bracket.bracket_visibility = True
        override(staff).tuplet_bracket.direction = Up
        override(staff).tuplet_bracket.padding = 1.25
        override(staff).tuplet_bracket.shorten_pair = (-1, -1.5)
        scheme = schemetools.Scheme('tuplet-number::calc-fraction-text')
        override(staff).tuplet_number.text = scheme
        set_(staff).tuplet_full_length = True
        layout_block = lilypondfiletools.Block(name='layout')
        layout_block.indent = 0
        layout_block.ragged_right = True
        score = scoretools.Score([staff])
        override(score).spacing_spanner.spacing_increment = 0.5
        set_(score).proportional_notation_duration = False
        return score, layout_block

    @staticmethod
    def _to_score_markup(selection):
        from abjad.tools import markuptools
        staff, layout_block = Duration._make_markup_score_block(selection)
        command = markuptools.MarkupCommand('score', [staff, layout_block])
        markup = markuptools.Markup(command)
        return markup

    ### PUBLIC PROPERTIES ###

    @property
    def dot_count(self):
        r'''Gets dot count.

        ..  container:: example

            **Example.** Gets dot count:

            ::

                >>> for n in range(1, 16 + 1):
                ...     try:
                ...         duration = Duration(n, 16)
                ...         sixteenths = duration.with_denominator(16)
                ...         dot_count = duration.dot_count
                ...         string = '{!s}\t{}'
                ...         string = string.format(sixteenths, dot_count)
                ...         print(string)
                ...     except AssignabilityError:
                ...         sixteenths = duration.with_denominator(16)
                ...         string = '{!s}\t{}'
                ...         string = string.format(sixteenths, '--')
                ...         print(string)
                ...
                1/16    0
                2/16    0
                3/16    1
                4/16    0
                5/16    --
                6/16    1
                7/16    2
                8/16    0
                9/16    --
                10/16   --
                11/16   --
                12/16   1
                13/16   --
                14/16   2
                15/16   3
                16/16   0

        Dot count defined equal to number of dots required to notate duration.

        Raises assignability error when duration is not assignable.

        Returns positive integer.
        '''
        if not self.is_assignable:
            raise AssignabilityError
        binary_string = mathtools.integer_to_binary_string(self.numerator)
        digit_sum = sum([int(x) for x in list(binary_string)])
        dot_count = digit_sum - 1
        return dot_count

    @property
    def equal_or_greater_assignable(self):
        r'''Gets assignable duration equal to or just greater than this
        duration.

        ..  container:: example

            **Example.** Gets equal-or-greater assignable duration:

            ::

                >>> for numerator in range(1, 16 + 1):
                ...     duration = Duration(numerator, 16)
                ...     result = duration.equal_or_greater_assignable
                ...     sixteenths = duration.with_denominator(16)
                ...     print('{!s}\t{!s}'.format(sixteenths, result))
                ...
                1/16    1/16
                2/16    1/8
                3/16    3/16
                4/16    1/4
                5/16    3/8
                6/16    3/8
                7/16    7/16
                8/16    1/2
                9/16    3/4
                10/16   3/4
                11/16   3/4
                12/16   3/4
                13/16   7/8
                14/16   7/8
                15/16   15/16
                16/16   1

        Returns new duration.
        '''
        good_denominator = mathtools.greatest_power_of_two_less_equal(
            self.denominator)
        current_numerator = self.numerator
        candidate = type(self)(current_numerator, good_denominator)
        while not candidate.is_assignable:
            current_numerator += 1
            candidate = type(self)(current_numerator, good_denominator)
        return candidate

    @property
    def equal_or_greater_power_of_two(self):
        r'''Gets duration equal or just greater power of two.

        ..  container:: example

            **Example.** Gets equal-or-greater power-of-two:

            ::

                >>> for numerator in range(1, 16 + 1):
                ...     duration = Duration(numerator, 16)
                ...     result = duration.equal_or_greater_power_of_two
                ...     sixteenths = duration.with_denominator(16)
                ...     print('{!s}\t{!s}'.format(sixteenths, result))
                ...
                1/16    1/16
                2/16    1/8
                3/16    1/4
                4/16    1/4
                5/16    1/2
                6/16    1/2
                7/16    1/2
                8/16    1/2
                9/16    1
                10/16   1
                11/16   1
                12/16   1
                13/16   1
                14/16   1
                15/16   1
                16/16   1

        Returns new duration.
        '''
        denominator_exponent = -int(math.ceil(math.log(self, 2)))
        return type(self)(1, 2) ** denominator_exponent

    @property
    def equal_or_lesser_assignable(self):
        r'''Gets assignable duration equal or just less than this duration.

        ..  container:: example

            **Example.** Gets equal-or-lesser assignable duration:

            ::

                >>> for numerator in range(1, 16 + 1):
                ...     duration = Duration(numerator, 16)
                ...     result = duration.equal_or_lesser_assignable
                ...     sixteenths = duration.with_denominator(16)
                ...     print('{!s}\t{!s}'.format(sixteenths, result))
                ...
                1/16    1/16
                2/16    1/8
                3/16    3/16
                4/16    1/4
                5/16    1/4
                6/16    3/8
                7/16    7/16
                8/16    1/2
                9/16    1/2
                10/16   1/2
                11/16   1/2
                12/16   3/4
                13/16   3/4
                14/16   7/8
                15/16   15/16
                16/16   1

        Returns new duration.
        '''
        good_denominator = mathtools.least_power_of_two_greater_equal(
            self.denominator)
        current_numerator = self.numerator
        candidate = type(self)(current_numerator, good_denominator)
        while not candidate.is_assignable:
            current_numerator -= 1
            candidate = type(self)(current_numerator, good_denominator)
        return candidate

    @property
    def equal_or_lesser_power_of_two(self):
        r'''Gets duration of the form ``d**2`` equal to or just less than this
        duration.

        ..  container:: example

            **Example.** Gets equal-or-lesser power-of-two:

            ::

                >>> for numerator in range(1, 16 + 1):
                ...     duration = Duration(numerator, 16)
                ...     result = duration.equal_or_lesser_power_of_two
                ...     sixteenths = duration.with_denominator(16)
                ...     print('{!s}\t{!s}'.format(sixteenths, result))
                ...
                1/16    1/16
                2/16    1/8
                3/16    1/8
                4/16    1/4
                5/16    1/4
                6/16    1/4
                7/16    1/4
                8/16    1/2
                9/16    1/2
                10/16   1/2
                11/16   1/2
                12/16   1/2
                13/16   1/2
                14/16   1/2
                15/16   1/2
                16/16   1

        Returns new duration.
        '''
        denominator_exponent = -int(math.floor(math.log(self, 2)))
        return type(self)(1, 2) ** denominator_exponent

    @property
    def flag_count(self):
        r'''Gets flag count.

        ..  container:: example

            **Example.** Gets flag count:

            ::

                >>> for n in range(1, 16 + 1):
                ...     duration = Duration(n, 64)
                ...     sixty_fourths = duration.with_denominator(64)
                ...     print('{!s}\t{}'.format(sixty_fourths, duration.flag_count))
                ...
                1/64    4
                2/64    3
                3/64    3
                4/64    2
                5/64    2
                6/64    2
                7/64    2
                8/64    1
                9/64    1
                10/64   1
                11/64   1
                12/64   1
                13/64   1
                14/64   1
                15/64   1
                16/64   0

        Flag count defined equal to number of flags required to notate
        duration.

        Returns nonnegative integer.
        '''
        # TODO: rewrite with only one operation per line
        flag_count = max(-int(math.floor(math.log(float(self.numerator) /
            self.denominator, 2))) - 2, 0)
        return flag_count

    @property
    def has_power_of_two_denominator(self):
        r'''Is true when duration is an integer power of two.
        Otherwise false.

        ..  container:: example

            **Example.** Is true when duration has power-of-two denominator:

            ::

                >>> for n in range(1, 16 + 1):
                ...     duration = Duration(1, n)
                ...     result = duration.has_power_of_two_denominator
                ...     print('{!s}\t{}'.format(duration, result))
                ...
                1       True
                1/2     True
                1/3     False
                1/4     True
                1/5     False
                1/6     False
                1/7     False
                1/8     True
                1/9     False
                1/10    False
                1/11    False
                1/12    False
                1/13    False
                1/14    False
                1/15    False
                1/16    True

        Returns true or false.
        '''
        exponent = math.log(self.denominator, 2)
        return int(exponent) == exponent

    @property
    def implied_prolation(self):
        r'''Gets implied prolation.

        ..  container:: example

            **Example.** Gets implied prolation:

            ::

                >>> for denominator in range(1, 16 + 1):
                ...     duration = Duration(1, denominator)
                ...     result = duration.implied_prolation
                ...     print('{!s}\t{!s}'.format(duration, result))
                ...
                1       1
                1/2     1
                1/3     2/3
                1/4     1
                1/5     4/5
                1/6     2/3
                1/7     4/7
                1/8     1
                1/9     8/9
                1/10    4/5
                1/11    8/11
                1/12    2/3
                1/13    8/13
                1/14    4/7
                1/15    8/15
                1/16    1

        Returns multipler.
        '''
        from abjad.tools import durationtools
        numerator = \
            mathtools.greatest_power_of_two_less_equal(self.denominator)
        return durationtools.Multiplier(numerator, self.denominator)

    @property
    def is_assignable(self):
        r'''Is true when duration is assignable. Otherwise false.

        ..  container:: example

            **Example.** Is true when duration is assignable:

            ::

                >>> for numerator in range(0, 16 + 1):
                ...     duration = Duration(numerator, 16)
                ...     sixteenths = duration.with_denominator(16)
                ...     print('{!s}\t{}'.format(sixteenths, duration.is_assignable))
                ...
                0/16    False
                1/16    True
                2/16    True
                3/16    True
                4/16    True
                5/16    False
                6/16    True
                7/16    True
                8/16    True
                9/16    False
                10/16   False
                11/16   False
                12/16   True
                13/16   False
                14/16   True
                15/16   True
                16/16   True

        Returns true or false.
        '''
        if 0 < self < 16:
            if mathtools.is_nonnegative_integer_power_of_two(
                self.denominator):
                if mathtools.is_assignable_integer(self.numerator):
                    return True
        return False

    @property
    def lilypond_duration_string(self):
        r'''Gets LilyPond duration string.

        ..  container:: example

            **Example.** Gets LilyPond duration string:

                >>> Duration(3, 16).lilypond_duration_string
                '8.'

        Raises assignability error when duration is not assignable.

        Returns string.
        '''
        if not self.is_assignable:
            raise AssignabilityError(self)
        undotted_rational = self.equal_or_lesser_power_of_two
        if undotted_rational <= 1:
            undotted_duration_string = str(undotted_rational.denominator)
        elif undotted_rational == type(self)(2, 1):
            undotted_duration_string = r'\breve'
        elif undotted_rational == type(self)(4, 1):
            undotted_duration_string = r'\longa'
        elif undotted_rational == type(self)(8, 1):
            undotted_duration_string = r'\maxima'
        else:
            message = 'can not process undotted rational: {}'
            message = message.format(undotted_rational)
            raise ValueError(message)
        dot_count = self.dot_count
        dot_string = '.' * dot_count
        dotted_duration_string = undotted_duration_string + dot_string
        return dotted_duration_string

    @property
    def pair(self):
        '''Gets numerator and denominator.

        ..  container:: example

            **Example.** Gets pair:

            ::

                >>> Duration(3, 16).pair
                (3, 16)

        Returns integer pair.
        '''
        return self.numerator, self.denominator

    @property
    def prolation_string(self):
        r'''Gets prolation string.

        ..  container:: example

            **Example.** Gets prolation string:

            ::

                >>> generator = Duration.yield_durations(unique=True)
                >>> for n in range(16):
                ...     duration = next(generator)
                ...     string = '{!s}\t{}'
                ...     string = string.format(duration, duration.prolation_string)
                ...     print(string)
                ...
                1       1:1
                2       1:2
                1/2     2:1
                1/3     3:1
                3       1:3
                4       1:4
                3/2     2:3
                2/3     3:2
                1/4     4:1
                1/5     5:1
                5       1:5
                6       1:6
                5/2     2:5
                4/3     3:4
                3/4     4:3
                2/5     5:2

        Returns string.
        '''
        return '{}:{}'.format(self.denominator, self.numerator)

    @property
    def reciprocal(self):
        '''Gets reciprocal.

        ..  container:: example

            **Example.** Gets reciprocal:

            ::

                >>> Duration(3, 7).reciprocal
                Duration(7, 3)

        Returns new duration.
        '''
        return type(self)(self.denominator, self.numerator)

    ### PUBLIC FUNCTIONS ###

    @staticmethod
    def durations_to_nonreduced_fractions(durations):
        r'''Changes `durations` to nonreduced fractions sharing
        least common denominator.

        ..  container:: example

            **Example.** Changes durations to nonreduced fractions:

            ::

                >>> durations = [Duration(2, 4), 3, (5, 16)]
                >>> result = Duration.durations_to_nonreduced_fractions(durations)
                >>> for x in result:
                ...     x
                ...
                NonreducedFraction(8, 16)
                NonreducedFraction(48, 16)
                NonreducedFraction(5, 16)

        Returns new object of `durations` type.
        '''
        durations = [Duration(x) for x in durations]
        denominators = [duration.denominator for duration in durations]
        lcd = mathtools.least_common_multiple(*denominators)
        nonreduced_fractions = [
            mathtools.NonreducedFraction(x).with_denominator(lcd)
            for x in durations
            ]
        result = type(durations)(nonreduced_fractions)
        return result

    @staticmethod
    def from_lilypond_duration_string(lilypond_duration_string):
        r'''Initializes duration from LilyPond duration string.

        ..  container:: example

            **Example.** Initializes duration from LilyPond duration string:

            ::

                >>> Duration.from_lilypond_duration_string('8.')
                Duration(3, 16)

        Returns duration.
        '''
        fraction = Duration._initialize_from_lilypond_duration_string(
            lilypond_duration_string)
        return Duration(fraction)

    @staticmethod
    def is_token(argument):
        '''Is true when `argument` correctly initializes a duration.
        Otherwise false.

        ..  container:: example

            **Example.** Is true when expression is a duration token:

            ::

                >>> Duration.is_token('8.')
                True

        Returns true or false.
        '''
        try:
            Duration.__new__(Duration, argument)
            return True
        except:
            return False

    def to_clock_string(self):
        r'''Changes duration to clock string.

        ..  container:: example

            **Example.** Changes duration to clock string:

            ::

                >>> note = Note("c'4")
                >>> duration = Duration(117)
                >>> clock_string = duration.to_clock_string()
                >>> clock_string
                "1'57''"

            ::

                >>> string = '"{}"'.format(clock_string)
                >>> markup = markuptools.Markup(string, direction=Up)
                >>> attach(markup, note)
                >>> show(note) # doctest: +SKIP

            ..  doctest::

                >>> print(format(note))
                c'4 ^ \markup { 1'57'' }

        Rounds down to nearest second.

        Returns string.
        '''
        minutes = int(self / 60)
        seconds = str(int(self - minutes * 60)).zfill(2)
        clock_string = "{}'{}''".format(minutes, seconds)
        return clock_string

    def to_score_markup(self):
        r'''Changes duration to score markup.

        ..  container:: example

            **Example 1.** Changes assignable duration to score markup:

            ::

                >>> markup = Duration(3, 16).to_score_markup()
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> f(markup)
                \markup {
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c'8.
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                    }

        ..  container:: example

            **Example 2.** Changes nonassignable duration to score markup:

            ::

                >>> markup = Duration(5, 16).to_score_markup()
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> f(markup)
                \markup {
                    \score
                        {
                            \new Score \with {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            } <<
                                \new RhythmicStaff \with {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                } {
                                    c'4 ~
                                    c'16
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                    }

        ..  container:: example

            **Example 3.** Override tuplet number text like this:

            ::

                >>> tuplet = Tuplet((5, 7), "c'16 c' c' c' c' c' c'")
                >>> attach(Beam(), tuplet[:])
                >>> staff = Staff([tuplet], context_name='RhythmicStaff')
                >>> duration = inspect_(tuplet).get_duration()
                >>> markup = duration.to_score_markup()
                >>> markup = markup.scale((0.75, 0.75))
                >>> override(tuplet).tuplet_number.text = markup
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new RhythmicStaff {
                    \override TupletNumber.text = \markup {
                        \scale
                            #'(0.75 . 0.75)
                            \score
                                {
                                    \new Score \with {
                                        \override SpacingSpanner.spacing-increment = #0.5
                                        proportionalNotationDuration = ##f
                                    } <<
                                        \new RhythmicStaff \with {
                                            \remove Time_signature_engraver
                                            \remove Staff_symbol_engraver
                                            \override Stem.direction = #up
                                            \override Stem.length = #5
                                            \override TupletBracket.bracket-visibility = ##t
                                            \override TupletBracket.direction = #up
                                            \override TupletBracket.padding = #1.25
                                            \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                            \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                            tupletFullLength = ##t
                                        } {
                                            c'4 ~
                                            c'16
                                        }
                                    >>
                                    \layout {
                                        indent = #0
                                        ragged-right = ##t
                                    }
                                }
                        }
                    \times 5/7 {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    \revert TupletNumber.text
                }

        Returns markup.
        '''
        from abjad.tools import scoretools
        notes = scoretools.make_leaves([0], [self])
        markup = self._to_score_markup(notes)
        return markup

    def with_denominator(self, denominator):
        r'''Changes duration to nonreduced fraction with `denominator`.

        ..  container:: example

            **Example.** Changes duration to nonreduced fraction:

            ::

                >>> duration = Duration(1, 4)
                >>> for denominator in (4, 8, 16, 32):
                ...     print(duration.with_denominator(denominator))
                ...
                1/4
                2/8
                4/16
                8/32

        Returns new duration.
        '''
        nonreduced_fraction = mathtools.NonreducedFraction(self)
        return nonreduced_fraction.with_denominator(denominator)

    @staticmethod
    def yield_durations(unique=False):
        r'''Yields all positive durations.

        ..  container:: example

            **Example 1.** Yields all positive durations in Cantor diagonalized
            order:

            ::

                >>> generator = Duration.yield_durations()
                >>> for n in range(16):
                ...     next(generator)
                ...
                Duration(1, 1)
                Duration(2, 1)
                Duration(1, 2)
                Duration(1, 3)
                Duration(1, 1)
                Duration(3, 1)
                Duration(4, 1)
                Duration(3, 2)
                Duration(2, 3)
                Duration(1, 4)
                Duration(1, 5)
                Duration(1, 2)
                Duration(1, 1)
                Duration(2, 1)
                Duration(5, 1)
                Duration(6, 1)

        ..  container:: example

            **Example 2.** Yields all positive durations in Cantor diagonalized
            order uniquely:

            ::

                >>> generator = Duration.yield_durations(unique=True)
                >>> for n in range(16):
                ...     next(generator)
                ...
                Duration(1, 1)
                Duration(2, 1)
                Duration(1, 2)
                Duration(1, 3)
                Duration(3, 1)
                Duration(4, 1)
                Duration(3, 2)
                Duration(2, 3)
                Duration(1, 4)
                Duration(1, 5)
                Duration(5, 1)
                Duration(6, 1)
                Duration(5, 2)
                Duration(4, 3)
                Duration(3, 4)
                Duration(2, 5)

        Returns generator.
        '''
        generator = mathtools.yield_nonreduced_fractions()
        while True:
            integer_pair = next(generator)
            duration = Duration(integer_pair)
            if not unique:
                yield duration
            elif duration.pair == integer_pair:
                yield duration

    def yield_equivalent_durations(self, minimum_written_duration=None):
        r'''Yields all durations equivalent to this duration.

        Returns output in Cantor diagonalized order.

        Ensures written duration never less than `minimum_written_duration`.

        ..  container:: example

            **Example 1.** Yields durations equivalent to ``1/8``:

            ::

                >>> pairs = Duration(1, 8).yield_equivalent_durations()
                >>> for pair in pairs: pair
                ...
                (Multiplier(1, 1), Duration(1, 8))
                (Multiplier(2, 3), Duration(3, 16))
                (Multiplier(4, 3), Duration(3, 32))
                (Multiplier(4, 7), Duration(7, 32))
                (Multiplier(8, 7), Duration(7, 64))
                (Multiplier(8, 15), Duration(15, 64))
                (Multiplier(16, 15), Duration(15, 128))
                (Multiplier(16, 31), Duration(31, 128))

        ..  container:: example

            **Example 2.** Yields durations equivalent to ``1/12``:

            ::

                >>> pairs = Duration(1, 12).yield_equivalent_durations()
                >>> for pair in pairs: pair
                ...
                (Multiplier(2, 3), Duration(1, 8))
                (Multiplier(4, 3), Duration(1, 16))
                (Multiplier(8, 9), Duration(3, 32))
                (Multiplier(16, 9), Duration(3, 64))
                (Multiplier(16, 21), Duration(7, 64))
                (Multiplier(32, 21), Duration(7, 128))
                (Multiplier(32, 45), Duration(15, 128))

        ..  container:: example

            Yields durations equivalent to ``5/48``:

            ::

                >>> pairs = Duration(5, 48).yield_equivalent_durations()
                >>> for pair in pairs: pair
                ...
                (Multiplier(5, 6), Duration(1, 8))
                (Multiplier(5, 3), Duration(1, 16))
                (Multiplier(5, 9), Duration(3, 16))
                (Multiplier(10, 9), Duration(3, 32))
                (Multiplier(20, 21), Duration(7, 64))
                (Multiplier(40, 21), Duration(7, 128))
                (Multiplier(8, 9), Duration(15, 128))

        Defaults `minimum_written_duration` to ``1/128``.

        Returns generator.
        '''
        if minimum_written_duration is None:
            minimum_written_duration = type(self)(1, 128)
        else:
            minimum_written_duration = type(self)(minimum_written_duration)
        generator = type(self).yield_durations(unique=True)
        pairs = []
        while True:
            written_duration = next(generator)
            if not written_duration.is_assignable:
                continue
            if written_duration < minimum_written_duration:
                pairs = tuple(pairs)
                return pairs
            prolation = self / written_duration
            if prolation.is_proper_tuplet_multiplier:
                pair = (prolation, written_duration)
                pairs.append(pair)



# -*- coding: utf-8 -*-
import importlib
import os


def yield_all_modules(
    code_root=None,
    ignored_directory_names=(
        '__pycache__',
        '.git',
        '.svn',
        'test',
        'docs',
        ),
    ignored_file_names=(
        '__init__.py',
        ),
    root_package_name=None,
    visit_private_modules=False,
    ):
    r'''Yields all modules encountered in `code_root whose name begins with
    `root_package_name`.

    Returns generator.
    '''
    from abjad import abjad_configuration
    if code_root is None:
        code_root = abjad_configuration.abjad_directory
    assert os.path.exists(code_root)
    if not os.path.exists(os.path.join(code_root, '__init__.py')):
        message = '{} is not a Python package directory.'
        message = message.format(code_root)
        raise ValueError(message)
    code_root = os.path.abspath(code_root)
    if root_package_name is None:
        parts = code_root.split(os.path.sep)
        root_package_name = parts[-1]
        while os.path.exists(os.path.join(
            os.path.sep.join(parts), '__init__.py')):
            root_package_name = parts.pop()
    visit_private_modules = bool(visit_private_modules)
    for current_root, directories, files in os.walk(code_root):
        # filter directories
        for directory in directories[:]:
            if directory in ignored_directory_names:
                directories.remove(directory)
            elif (directory.startswith('_') and
                not visit_private_modules):
                directories.remove(directory)
            elif not os.path.exists(os.path.join(
                current_root, directory, '__init__.py')):
                directories.remove(directory)
        directories.sort()
        # filter files
        for file_name in files[:]:
            if file_name in ignored_file_names:
                files.remove(file_name)
            elif not file_name.endswith('.py'):
                files.remove(file_name)
        files.sort()
        # process files
        for file_name in files:
            if file_name == '__init__.py':
                path = current_root
            else:
                path = os.path.join(current_root, file_name).replace('.py', '')
            parts = path.split(os.path.sep)
            module_name = []
            for part in reversed(parts):
                module_name.append(part)
                if part == root_package_name:
                    break
            module_name = '.'.join(reversed(module_name))
            module = importlib.import_module(module_name)
            #module = __import__(module_name, fromlist=['*'])
            yield module



# -*- coding: utf-8 -*-
import os
import subprocess
import sys
from docutils import nodes
from sphinx.util.compat import Directive
from sphinx.util.nodes import set_source_info


class ShellDirective(Directive):
    r'''An abjad-book shell directive.

    Represents a shell session.

    Generates a docutils ``literal_block`` node.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Sphinx Internals'

    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = False
    option_spec = {}

    ### PRIVATE METHODS ###

    def _read_from_pipe(self, pipe, strip=True):
        lines = []
        string = pipe.read()
        if sys.version_info[0] == 2:
            for line in string.splitlines():
                line = str(line)
                if strip:
                    line = line.strip()
                lines.append(line)
        else:
            for line in string.splitlines():
                line = line.decode('utf-8')
                if strip:
                    line = line.strip()
                lines.append(line)
        return '\n'.join(lines)

    ### PUBLIC METHODS ###

    def run(self):
        r'''Executes the directive.
        '''
        from abjad import abjad_configuration
        self.assert_has_content()
        os.chdir(abjad_configuration.abjad_directory)
        result = []
        for line in self.content:
            curdir = os.path.basename(os.path.abspath(os.path.curdir))
            prompt = '{}$ '.format(curdir)
            prompt += line
            result.append(prompt)
            process = subprocess.Popen(
                line,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                )
            stdout = self._read_from_pipe(process.stdout)
            stderr = self._read_from_pipe(process.stderr)
            result.append(stdout)
            result.append(stderr)
        code = '\n'.join(result)
        literal = nodes.literal_block(code, code)
        literal['language'] = 'console'
        set_source_info(self, literal)
        return [literal]



# -*- coding: utf-8 -*-
from abjad import Fraction
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class NonreducedFraction(AbjadObject, Fraction):
    r'''Nonereduced fraction.

    ..  container:: example

        Initializes with an integer numerator and integer denominator:

        ::

            >>> NonreducedFraction(3, 6)
            NonreducedFraction(3, 6)

    ..  container:: example

        Initializes with only an integer denominator:

        ::

            >>> NonreducedFraction(3)
            NonreducedFraction(3, 1)

    ..  container:: example

        Initializes with an integer pair:

        ::

            >>> NonreducedFraction((3, 6))
            NonreducedFraction(3, 6)

    ..  container:: example

        Initializes with an integer singleton:

        ::

            >>> NonreducedFraction((3,))
            NonreducedFraction(3, 1)

    ..  container:: example

        Nonreduced fractions inherit from built-in fraction:

        ::

            >>> isinstance(NonreducedFraction(3, 6), Fraction)
            True

    ..  container:: example

        Nonreduced fractions are numbers:

        ::

            >>> import numbers

        ::

            >>> isinstance(NonreducedFraction(3, 6), numbers.Number)
            True

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_numerator',
        '_denominator',
        )

    ### CONSTRUCTOR ###

    def __new__(class_, *arguments):
        from abjad.tools import mathtools
        if (len(arguments) == 1 and hasattr(arguments[0], 'numerator') and
            hasattr(arguments[0], 'denominator')):
            numerator = arguments[0].numerator
            denominator = arguments[0].denominator
        elif len(arguments) == 1 and isinstance(arguments[0], int):
            numerator = arguments[0]
            denominator = 1
        elif len(arguments) == 1 and mathtools.is_integer_singleton(arguments[0]):
            numerator = arguments[0][0]
            denominator = 1
        elif len(arguments) == 1 and mathtools.is_integer_pair(arguments[0]):
            numerator, denominator = arguments[0]
        elif len(arguments) == 1 and isinstance(arguments[0], str):
            numerator, denominator = class_._parse_input_string(arguments[0])
        elif mathtools.is_integer_pair(arguments):
            numerator = arguments[0]
            denominator = arguments[1]
        elif len(arguments) == 0:
            numerator = 0
            denominator = 1
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(class_.__name__, arguments)
            raise ValueError(message)
        numerator *= mathtools.sign(denominator)
        denominator = abs(denominator)
        self = Fraction.__new__(class_, numerator, denominator)
        self._numerator = numerator
        self._denominator = denominator
        return self

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Gets absolute value of nonreduced fraction.

        ..  container:: example

            ::

                >>> abs(NonreducedFraction(-3, 3))
                NonreducedFraction(3, 3)

        Returns nonreduced fraction.
        '''
        pair = (abs(self.numerator), self.denominator)
        return self._from_pair(pair)

    def __add__(self, argument):
        r'''Adds `argument` to nonreduced fraction.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) + 1
                NonreducedFraction(6, 3)

            ::

                >>> 1 + NonreducedFraction(3, 3)
                NonreducedFraction(6, 3)

        Returns nonreduced fraction.
        '''
        from abjad.tools import mathtools
        if isinstance(argument, int):
            numerator = self.numerator + argument * self.denominator
            pair = (numerator, self.denominator)
            return self._from_pair(pair)
        elif hasattr(argument, 'denominator'):
            if self.denominator == argument.denominator:
                numerator = self.numerator + argument.numerator
                pair = (numerator, self.denominator)
                return self._from_pair(pair)
            else:
                denominators = [self.denominator, argument.denominator]
                denominator = mathtools.least_common_multiple(*denominators)
                self_multiplier = denominator // self.denominator
                argument_multiplier = denominator // argument.denominator
                self_numerator = self_multiplier * self.numerator
                argument_numerator = argument_multiplier * argument.numerator
                pair = (self_numerator + argument_numerator, denominator)
                return self._from_pair(pair)
        else:
            raise ValueError(argument)

    def __div__(self, argument):
        r'''Divides nonreduced fraction by `argument`.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) / 1
                NonreducedFraction(3, 3)

        Returns nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(argument, type(self)):
            denominators.append(argument.denominator)
            argument = argument.reduce()
        fraction = self.reduce() / argument
        return self._fraction_with_denominator(fraction, max(denominators))

    def __eq__(self, argument):
        r'''Is true when `argument` equals nonreduced fraction.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) == 1
                True

        Returns true or false.
        '''
        return self.reduce() == argument

    def __format__(self, format_specification=''):
        r'''Formats nonreduced fraction.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            ::

                >>> fraction = NonreducedFraction(-6, 3)
                >>> print(format(fraction))
                mathtools.NonreducedFraction(-6, 3)

        Returns string.
        '''
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __ge__(self, argument):
        r'''Is true when nonreduced fraction is greater than or equal to
        `argument`.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) >= 1
                True

        Returns true or false.
        '''
        return self.reduce() >= argument

    def __gt__(self, argument):
        r'''Is true when nonreduced fraction is greater than `argument`.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) > 1
                False

        Returns true or false.
        '''
        return self.reduce() > argument

    def __hash__(self):
        r'''Hashes nonreduced fraction.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        superclass = super(NonreducedFraction, self)
        return superclass.__hash__()

    def __le__(self, argument):
        r'''Is true when nonreduced fraction is less than or equal to `argument`.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) <= 1
                True

        Returns true or false.
        '''
        return self.reduce() <= argument

    def __lt__(self, argument):
        r'''Is true when nonreduced fraction is less than `argument`.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) < 1
                False

        Returns true or false.
        '''
        return self.reduce() < argument

    def __mul__(self, argument):
        r'''Multiplies nonreduced fraction by `argument`.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) * 3
                NonreducedFraction(9, 3)

        Returns nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(argument, type(self)):
            denominators.append(argument.denominator)
            argument = argument.reduce()
        fraction = self.reduce() * argument
        return self._fraction_with_denominator(fraction, max(denominators))

    def __ne__(self, argument):
        r'''Is true when `argument` does not equal nonreduced fraction.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) != 'foo'
                True

        Returns true or false.
        '''
        return not self == argument

    def __neg__(self):
        r'''Negates nonreduced fraction.

        ..  container:: example

            ::

                >>> -NonreducedFraction(3, 3)
                NonreducedFraction(-3, 3)

        Returns nonreduced fraction.
        '''
        pair = (-self.numerator, self.denominator)
        return self._from_pair(pair)

    def __pow__(self, argument):
        r'''Raises nonreduced fraction to `argument`.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 6) ** -1
                NonreducedFraction(6, 3)

        Returns nonreduced fraction.
        '''
        if argument == -1:
            pair = (self.denominator, self.numerator)
            return self._from_pair(pair)
        superclass = super(NonreducedFraction, self)
        return superclass.__pow__(argument)

    def __radd__(self, argument):
        r'''Adds nonreduced fraction to `argument`.

        ..  container:: example

            ::

                >>> 1 + NonreducedFraction(3, 3)
                NonreducedFraction(6, 3)

        Returns nonreduced fraction.
        '''
        return self + argument

    def __rdiv__(self, argument):
        r'''Divides `argument` by nonreduced fraction.

        ..  container:: example

            ::

                >>> 1 / NonreducedFraction(3, 3)
                NonreducedFraction(3, 3)

        Returns nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(argument, type(self)):
            denominators.append(argument.denominator)
            argument = argument.reduce()
        fraction = argument / self.reduce()
        return self._fraction_with_denominator(fraction, max(denominators))

    __rtruediv__ = __rdiv__

    def __repr__(self):
        r'''Gets interpreter representation of nonreduced fraction.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 6)
                NonreducedFraction(3, 6)

        Returns string.
        '''
        return systemtools.StorageFormatAgent(self).get_repr_format()

    def __rmul__(self, argument):
        r'''Multiplies `argument` by nonreduced fraction.

        ..  container:: example

            ::

                >>> 3 * NonreducedFraction(3, 3)
                NonreducedFraction(9, 3)

        Returns nonreduced fraction.
        '''
        return self * argument

    def __rsub__(self, argument):
        r'''Subtracts nonreduced fraction from `argument`.

        ..  container:: example

            ::

                >>> 1 - NonreducedFraction(3, 3)
                NonreducedFraction(0, 3)

        Returns nonreduced fraction.
        '''
        return -self + argument

    def __str__(self):
        r'''Gets string representation of nonreduced fraction.

        ..  container:: example

            ::

                >>> fraction = NonreducedFraction(-6, 3)

            ::

                >>> str(fraction)
                '-6/3'

        Returns string.
        '''
        return '{}/{}'.format(self.numerator, self.denominator)

    def __sub__(self, argument):
        r'''Subtracts `argument` from nonreduced fraction.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 3) - 2
                NonreducedFraction(-3, 3)

            ::

                >>> NonreducedFraction(5, 4) - NonreducedFraction(18, 16)
                NonreducedFraction(2, 16)

            ::

                >>> NonreducedFraction(18, 16) - NonreducedFraction(5, 4)
                NonreducedFraction(-2, 16)

        Returns nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(argument, type(self)):
            denominators.append(argument.denominator)
            argument = argument.reduce()
        fraction = self.reduce() - argument
        return self._fraction_with_denominator(fraction, max(denominators))

    def __truediv__(self, argument):
        r'''Divides nonreduced fraction in Python 3.

        Returns nonreduced fraction.
        '''
        return self.__div__(argument)

    ### PRIVATE METHODS ###

    def _fraction_with_denominator(self, fraction, denominator):
        from abjad.tools import mathtools
        denominators = [denominator, fraction.denominator]
        denominator = mathtools.least_common_multiple(*denominators)
        result = self._from_pair(fraction)
        result = result.with_denominator(denominator)
        return result

    def _from_pair(self, pair):
        r'''Method is designed to be subclassed.
        '''
        return type(self)(pair)

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_args_values=[
                self.numerator,
                self.denominator,
                ],
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            )

    @staticmethod
    def _parse_input_string(string):
        if '/' in string:
            numerator, denominator = string.split('/')
            numerator = int(numerator)
            denominator = int(denominator)
        else:
            numerator = int(string)
            denominator = 1
        return numerator, denominator

    ### PUBLIC METHODS ###

    def multiply_with_cross_cancelation(self, multiplier):
        '''Multiplies nonreduced fraction by `argument` with cross-cancelation.

        ..  container:: example

            ::

                >>> fraction = NonreducedFraction(4, 8)

            ::

                >>> fraction.multiply_with_cross_cancelation((2, 3))
                NonreducedFraction(4, 12)

            ::

                >>> fraction.multiply_with_cross_cancelation((4, 1))
                NonreducedFraction(4, 2)

            ::

                >>> fraction.multiply_with_cross_cancelation((3, 5))
                NonreducedFraction(12, 40)

            ::

                >>> fraction.multiply_with_cross_cancelation((6, 5))
                NonreducedFraction(12, 20)

            ::

                >>> fraction = NonreducedFraction(5, 6)
                >>> fraction.multiply_with_cross_cancelation((6, 5))
                NonreducedFraction(1, 1)

        Returns nonreduced fraction.
        '''
        from abjad.tools import durationtools
        from abjad.tools import mathtools
        multiplier = durationtools.Multiplier(multiplier)
        self_numerator_factors = mathtools.factors(self.numerator)
        multiplier_denominator_factors = mathtools.factors(
            multiplier.denominator)
        for factor in multiplier_denominator_factors[:]:
            if factor in self_numerator_factors:
                self_numerator_factors.remove(factor)
                multiplier_denominator_factors.remove(factor)
        self_denominator_factors = mathtools.factors(self.denominator)
        multiplier_numerator_factors = mathtools.factors(multiplier.numerator)
        for factor in multiplier_numerator_factors[:]:
            if factor in self_denominator_factors:
                self_denominator_factors.remove(factor)
                multiplier_numerator_factors.remove(factor)
        result_numerator_factors = self_numerator_factors + \
            multiplier_numerator_factors
        result_denominator_factors = self_denominator_factors + \
            multiplier_denominator_factors
        result_numerator = 1
        for factor in result_numerator_factors:
            result_numerator *= factor
        result_denominator = 1
        for factor in result_denominator_factors:
            result_denominator *= factor
        pair = (result_numerator, result_denominator)
        return self._from_pair(pair)

    def multiply_with_numerator_preservation(self, multiplier):
        r'''Multiplies nonreduced fraction by `multiplier` with numerator
        preservation where possible.

        ..  container:: example

            ::

                >>> fraction = NonreducedFraction(9, 16)

            ::

                >>> fraction.multiply_with_numerator_preservation((2, 3))
                NonreducedFraction(9, 24)

            ::

                >>> fraction.multiply_with_numerator_preservation((1, 2))
                NonreducedFraction(9, 32)

            ::

                >>> fraction.multiply_with_numerator_preservation((5, 6))
                NonreducedFraction(45, 96)

            ::

                >>> fraction = NonreducedFraction(3, 8)

            ::

                >>> fraction.multiply_with_numerator_preservation((2, 3))
                NonreducedFraction(3, 12)

        Returns nonreduced fraction.
        '''
        from abjad.tools import durationtools
        multiplier = durationtools.Multiplier(multiplier)
        self_denominator = self.denominator
        candidate_result_denominator = self_denominator / multiplier
        if candidate_result_denominator.denominator == 1:
            pair = (self.numerator, candidate_result_denominator.numerator)
            return self._from_pair(pair)
        else:
            result_numerator = \
                self.numerator * candidate_result_denominator.denominator
            result_denominator = candidate_result_denominator.numerator
            pair = (result_numerator, result_denominator)
            return self._from_pair(pair)

    def multiply_without_reducing(self, argument):
        r'''Multiplies nonreduced fraction by `argument` without reducing.

        ..  container:: example

            ::

                >>> fraction = NonreducedFraction(3, 8)

            ::

                >>> fraction.multiply_without_reducing((3, 3))
                NonreducedFraction(9, 24)

            ::

                >>> fraction = NonreducedFraction(4, 8)

            ::

                >>> fraction.multiply_without_reducing((4, 5))
                NonreducedFraction(16, 40)

            ::

                >>> fraction.multiply_without_reducing((3, 4))
                NonreducedFraction(12, 32)

        Returns nonreduced fraction.
        '''
        argument = self._from_pair(argument)
        numerator = self.numerator * argument.numerator
        denominator = self.denominator * argument.denominator
        pair = (numerator, denominator)
        return self._from_pair(pair)

    def reduce(self):
        r'''Reduces nonreduced fraction.

        ..  container:: example

            ::

                >>> fraction = NonreducedFraction(-6, 3)

            ::

                >>> fraction.reduce()
                Fraction(-2, 1)

        Returns fraction.
        '''
        return Fraction(self.numerator, self.denominator)

    def with_denominator(self, denominator):
        r'''Returns new nonreduced fraction with integer `denominator`.

        ..  container:: example

            ::

                >>> NonreducedFraction(3, 6).with_denominator(12)
                NonreducedFraction(6, 12)

        ..  container:: example

            ::

                >>> for numerator in range(12):
                ...     fraction = NonreducedFraction(numerator, 6)
                ...     print(fraction, fraction.with_denominator(12))
                ...
                0/6 0/12
                1/6 2/12
                2/6 4/12
                3/6 6/12
                4/6 8/12
                5/6 10/12
                6/6 12/12
                7/6 14/12
                8/6 16/12
                9/6 18/12
                10/6 20/12
                11/6 22/12

            ::

                >>> for numerator in range(12):
                ...     fraction = NonreducedFraction(numerator, 6)
                ...     print(fraction, fraction.with_denominator(8))
                ...
                0/6 0/8
                1/6 1/6
                2/6 2/6
                3/6 4/8
                4/6 4/6
                5/6 5/6
                6/6 8/8
                7/6 7/6
                8/6 8/6
                9/6 12/8
                10/6 10/6
                11/6 11/6

            ::

                >>> for numerator in range(12):
                ...     fraction = NonreducedFraction(numerator, 6)
                ...     print(fraction, fraction.with_denominator(12))
                ...
                0/6 0/12
                1/6 2/12
                2/6 4/12
                3/6 6/12
                4/6 8/12
                5/6 10/12
                6/6 12/12
                7/6 14/12
                8/6 16/12
                9/6 18/12
                10/6 20/12
                11/6 22/12

        Returns nonreduced fraction.
        '''
        from abjad.tools import durationtools
        current_numerator, current_denominator = self.pair
        multiplier = durationtools.Multiplier(denominator, current_denominator)
        new_numerator = multiplier * current_numerator
        new_denominator = multiplier * current_denominator
        if (new_numerator.denominator == 1 and
            new_denominator.denominator == 1):
            pair = (new_numerator.numerator, new_denominator.numerator)
        else:
            pair = (current_numerator, current_denominator)
        return self._from_pair(pair)

    def with_multiple_of_denominator(self, denominator):
        r'''Returns new nonreduced fraction with multiple of integer
        `denominator`.

        ..  container:: example

            ::

                >>> fraction = NonreducedFraction(3, 6)

            ::

                >>> fraction.with_multiple_of_denominator(5)
                NonreducedFraction(5, 10)

        ..  container:: example

            ::

                >>> NonreducedFraction(1, 2).with_multiple_of_denominator(2)
                NonreducedFraction(1, 2)

            ::

                >>> NonreducedFraction(1, 2).with_multiple_of_denominator(4)
                NonreducedFraction(2, 4)

            ::

                >>> NonreducedFraction(1, 2).with_multiple_of_denominator(8)
                NonreducedFraction(4, 8)

            ::

                >>> NonreducedFraction(1, 2).with_multiple_of_denominator(16)
                NonreducedFraction(8, 16)

        Returns nonreduced fraction.
        '''
        result = self.with_denominator(denominator)
        while not result.denominator == denominator:
            denominator *= 2
            result = result.with_denominator(denominator)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def denominator(self):
        r'''Denominator of nonreduced fraction.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.denominator
            3

        Returns positive integer.
        '''
        return self._denominator

    @property
    def imag(self):
        r'''Nonreduced fractions have no imaginary part.

        ::

            >>> fraction.imag
            0

        Returns zero.
        '''
        return 0

    @property
    def numerator(self):
        r'''Numerator of nonreduced fraction.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.numerator
            -6

        Returns integer.
        '''
        return self._numerator

    @property
    def pair(self):
        r'''Read only pair of nonreduced fraction numerator and denominator.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.pair
            (-6, 3)

        Returns integer pair.
        '''
        return self.numerator, self.denominator

    @property
    def real(self):
        r'''Nonreduced fractions are their own real component.

        ::

            >>> fraction.real
            NonreducedFraction(-6, 3)

        Returns nonreduced fraction.
        '''
        return self
